@@ -10,10 +10,10 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	6 June 1994
- * Last Edited:	24 October 2000
+ * Last Edited:	27 August 2001
  * 
  * The IMAP toolkit provided in this Distribution is
- * Copyright 2000 University of Washington.
+ * Copyright 2001 University of Washington.
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this Distribution.
  */
@@ -84,8 +84,7 @@ MAILSTREAM pop3proto = {&pop3driver};
 				/* driver parameters */
 static unsigned long pop3_maxlogintrials = MAXLOGINTRIALS;
 static long pop3_port = 0;
-static long pop3_altport = 0;
-static char *pop3_altname = NIL;
+static long pop3_sslport = 0;
 
 /* POP3 mail validate mailbox
  * Accepts: mailbox name
@@ -124,17 +123,11 @@ void *pop3_parameters (long function,void *value)
   case GET_POP3PORT:
     value = (void *) pop3_port;
     break;
-  case SET_ALTPOPPORT:
-    pop3_altport = (long) value;
+  case SET_SSLPOPPORT:
+    pop3_sslport = (long) value;
     break;
-  case GET_ALTPOPPORT:
-    value = (void *) pop3_altport;
-    break;
-  case SET_ALTPOPNAME:
-    pop3_altname = (char *) value;
-    break;
-  case GET_ALTPOPNAME:
-    value = (void *) pop3_altname;
+  case GET_SSLPOPPORT:
+    value = (void *) pop3_sslport;
     break;
   default:
     value = NIL;		/* error case */
@@ -317,37 +310,40 @@ MAILSTREAM *pop3_open (MAILSTREAM *stream)
   if (mb.anoflag || stream->anonymous) {
     mm_log ("Anonymous POP3 login not available",ERROR);
     return NIL;
+  }
+				/* /readonly not supported either */
+  if (mb.readonlyflag || stream->rdonly) {
+    mm_log ("Read-only POP3 access not available",ERROR);
+    return NIL;
   }
 				/* copy other switches */
   if (mb.dbgflag) stream->debug = T;
   if (mb.secflag) stream->secure = T;
-  mb.tryaltflag = stream->tryalt;
-  stream->local = fs_get (sizeof (POP3LOCAL));
+  mb.trysslflag = stream->tryssl = (mb.trysslflag || stream->tryssl) ? T : NIL;
+  stream->local =		/* instantiate localdata */
+    (void *) memset (fs_get (sizeof (POP3LOCAL)),0,sizeof (POP3LOCAL));
   stream->sequence++;		/* bump sequence number */
   stream->perm_deleted = T;	/* deleted is only valid flag */
-  LOCAL->response = LOCAL->reply = NIL;
-				/* currently no message */
-  LOCAL->msgno = LOCAL->hdrsize = 0;
-  LOCAL->txt = NIL;		/* no file initially */
 
   if ((LOCAL->netstream =	/* try to open connection */
        net_open (&mb,NIL,pop3_port ? pop3_port : POP3TCPPORT,
-		 (NETDRIVER *) mail_parameters (NIL,GET_ALTDRIVER,NIL),
-		 (char *) mail_parameters (NIL,GET_ALTPOPNAME,NIL),
-		 (unsigned long) mail_parameters (NIL,GET_ALTPOPPORT,NIL))) &&
+		 (NETDRIVER *) mail_parameters (NIL,GET_SSLDRIVER,NIL),
+		 "*pop3s",pop3_sslport ? pop3_sslport : POP3SSLPORT)) &&
       pop3_reply (stream)) {
     mm_log (LOCAL->reply,NIL);	/* give greeting */
     if (!pop3_auth (stream,&mb,tmp,usr)) pop3_close (stream,NIL);
     else if (pop3_send (stream,"STAT",NIL)) {
       int silent = stream->silent;
       stream->silent = T;
-      sprintf (tmp,"{%.200s:%lu/pop3",net_host (LOCAL->netstream),
+      sprintf (tmp,"{%.200s:%lu/pop3",
+	       (int) mail_parameters (NIL,GET_TRUSTDNS,NIL) ?
+	       net_host (LOCAL->netstream) : mb.host,
 	       net_port (LOCAL->netstream));
-      if (mb.altflag) sprintf (tmp + strlen (tmp),"/%.200s",(char *)
-			       mail_parameters (NIL,GET_ALTDRIVERNAME,NIL));
-      if (mb.altopt) sprintf (tmp + strlen (tmp),"/%.200s",(char *)
-			      mail_parameters (NIL,GET_ALTOPTIONNAME,NIL));
-      if (mb.secflag) strcat (tmp,"/secure");
+      if (mb.tlsflag) strcat (tmp,"/tls");
+      if (mb.notlsflag) strcat (tmp,"/notls");
+      if (mb.sslflag) strcat (tmp,"/ssl");
+      if (mb.novalidate) strcat (tmp,"/novalidate-cert");
+      if (stream->secure) strcat (tmp,"/secure");
       sprintf (tmp + strlen (tmp),"/user=\"%s\"}%s",usr,mb.mailbox);
       stream->inbox = T;	/* always INBOX */
       fs_give ((void **) &stream->mailbox);
@@ -378,27 +374,124 @@ MAILSTREAM *pop3_open (MAILSTREAM *stream)
   return LOCAL ? stream : NIL;	/* if stream is alive, return to caller */
 }
 
+/* POP3 capabilities
+ * Accepts: stream
+ *	    authenticator flags
+ * Returns: T on success, NIL on failure
+ */
+
+long pop3_capa (MAILSTREAM *stream,long flags)
+{
+  unsigned long i;
+  char *s,*t,*args;
+  if (LOCAL->cap.implementation)/* zap all old capabilities */
+    fs_give ((void **) &LOCAL->cap.implementation);
+  memset (&LOCAL->cap,0,sizeof (LOCAL->cap));
+				/* get server capabilities */
+  if (!pop3_send (stream,"CAPA",NIL)) {
+				/* guess at them */
+    LOCAL->cap.top = LOCAL->cap.uidl = LOCAL->cap.user = T;
+    return NIL;			/* no CAPA on this server */
+  }
+  while ((t = net_getline (LOCAL->netstream)) && (t[1] || (*t != '.'))) {
+    if (stream->debug) mm_dlog (t);
+				/* get optional capability arguments */
+    if (args = strchr (t,' ')) *args++ = '\0';
+    if (!compare_cstring (t,"STLS")) LOCAL->cap.stls = T;
+    else if (!compare_cstring (t,"PIPELINING")) LOCAL->cap.pipelining = T;
+    else if (!compare_cstring (t,"RESP-CODES")) LOCAL->cap.respcodes = T;
+    else if (!compare_cstring (t,"TOP")) LOCAL->cap.top = T;
+    else if (!compare_cstring (t,"UIDL")) LOCAL->cap.uidl = T;
+    else if (!compare_cstring (t,"USER")) LOCAL->cap.user = T;
+    else if (!compare_cstring (t,"IMPLEMENTATION") && args)
+      LOCAL->cap.implementation = cpystr (args);
+    else if (!compare_cstring (t,"EXPIRE") && args) {
+      LOCAL->cap.expire = T;	/* note that it is present */
+      if (s = strchr(args,' ')){/* separate time from possible USER */
+	*s++ = '\0';
+				/* in case they add something after USER */
+	if ((strlen (s) > 4) && (s[4] == ' ')) s[4] = '\0';
+      }
+				/* infinite expiration */
+      if (!compare_cstring (args,"NEVER")) LOCAL->cap.expire = 65535;
+      else {			/* in case they add something after USER */
+	if ((strlen (s) > 4) && (s[4] == ' ')) s[4] = '\0';
+				/* get delay time */
+	LOCAL->cap.expire = (s && !compare_cstring (s,"USER")) ?
+	  -atoi (args) : atoi (args);
+      }
+    }
+    else if (!compare_cstring (t,"LOGIN-DELAY") && args) {
+      LOCAL->cap.logindelay = T;/* note that it is present */
+      if (s = strchr(args,' ')){/* separate time from possible USER */
+	*s++ = '\0';
+				/* in case they add something after USER */
+	if ((strlen (s) > 4) && (s[4] == ' ')) s[4] = '\0';
+      }
+				/* get delay time */
+      LOCAL->cap.delaysecs = (s && !compare_cstring (s,"USER")) ?
+	-atoi (args) : atoi (args);
+    }
+    else if (!compare_cstring (t,"SASL") && args)
+      for (args = strtok (args," "); args; args = strtok (NIL," "))
+	if ((i = mail_lookup_auth_name (args,flags)) &&
+	    (--i < MAXAUTHENTICATORS))
+	  LOCAL->cap.sasl |= (1 << i);
+    fs_give ((void **) &t);
+  }
+  if (t) {			/* flush end of text indicator */
+    if (stream->debug) mm_dlog (t);
+    fs_give ((void **) &t);
+  }
+  return LONGT;
+}
+
 /* POP3 authenticate
  * Accepts: stream to login
  *	    parsed network mailbox structure
- *	    scratch buffer
+ *	    scratch buffer of length MAILTMPLEN
  *	    place to return user name
  * Returns: T on success, NIL on failure
  */
 
-long pop3_auth (MAILSTREAM *stream,NETMBX *mb,char *tmp,char *usr)
+long pop3_auth (MAILSTREAM *stream,NETMBX *mb,char *pwd,char *usr)
 {
   unsigned long i,trial,auths = 0;
   char *t;
   AUTHENTICATOR *at;
+  long ret = NIL;
   long flags = (stream->secure ? AU_SECURE : NIL) |
     (mb->authuser[0] ? AU_AUTHUSER : NIL);
+  long capaok = pop3_capa (stream,flags);
+  NETDRIVER *ssld = (NETDRIVER *) mail_parameters (NIL,GET_SSLDRIVER,NIL);
+  sslstart_t stls = (sslstart_t) mail_parameters (NIL,GET_SSLSTART,NIL);
+				/* server has TLS? */
+  if (stls && LOCAL->cap.stls && !mb->sslflag && !mb->notlsflag &&
+      pop3_send (stream,"STLS",NIL)) {
+    mb->tlsflag = T;		/* TLS OK, get into TLS at this end */
+    LOCAL->netstream->dtb = ssld;
+    if (!(LOCAL->netstream->stream =
+	  (*stls) (LOCAL->netstream->stream,mb->host,NET_TLSCLIENT |
+		   (mb->novalidate ? NET_NOVALIDATECERT : NIL)))) {
+				/* drat, drop this connection */
+      if (LOCAL->netstream) net_close (LOCAL->netstream);
+      LOCAL->netstream= NIL;
+      return NIL;		/* TLS negotiation failed */
+    }
+    pop3_capa (stream,flags);	/* get capabilities now that TLS in effect */
+  }
+  else if (mb->tlsflag) {	/* user specified /tls but can't do it */
+    mm_log ("Unable to negotiate TLS with this server",ERROR);
+    return NIL;
+  }
+  				/* get authenticators from capabilities */
+  if (capaok) auths = LOCAL->cap.sasl;
 				/* get list of authenticators */
-  if (pop3_send (stream,"AUTH",NIL)) {
+  else if (pop3_send (stream,"AUTH",NIL)) {
     while ((t = net_getline (LOCAL->netstream)) && (t[1] || (*t != '.'))) {
       if (stream->debug) mm_dlog (t);
-      if ((i = mail_lookup_auth_name (t,flags)) &&
-	  (--i < (8*sizeof (unsigned long)))) auths |= (1 << i);
+      if ((i = mail_lookup_auth_name (t,flags)) && (--i < MAXAUTHENTICATORS))
+	auths |= (1 << i);
       fs_give ((void **) &t);
     }
     if (t) {			/* flush end of text indicator */
@@ -406,60 +499,81 @@ long pop3_auth (MAILSTREAM *stream,NETMBX *mb,char *tmp,char *usr)
       fs_give ((void **) &t);
     }
   }
+				/* disable LOGIN if PLAIN also advertised */
+  if ((i = mail_lookup_auth_name ("PLAIN",NIL)) && (--i < MAXAUTHENTICATORS) &&
+      (auths & (1 << i)) &&
+      (i = mail_lookup_auth_name ("LOGIN",NIL)) && (--i < MAXAUTHENTICATORS))
+    auths &= ~(1 << i);
 
   if (auths) {			/* got any authenticators? */
-    for (t = NIL; LOCAL->netstream && auths &&
+    if ((int) mail_parameters (NIL,GET_TRUSTDNS,NIL)) {
+				/* remote name for authentication */
+      strncpy (mb->host,(int) mail_parameters (NIL,GET_SASLUSESPTRNAME,NIL) ?
+	       net_remotehost (LOCAL->netstream) : net_host (LOCAL->netstream),
+	       NETMAXHOST-1);
+      mb->host[NETMAXHOST-1] = '\0';
+    }
+    for (t = NIL; !ret && LOCAL->netstream && auths &&
 	 (at = mail_lookup_auth (find_rightmost_bit (&auths)+1)); ) {
       if (t) {			/* previous authenticator failed? */
-	sprintf (tmp,"Retrying using %.80s authentication after %.80s",
+	sprintf (pwd,"Retrying using %.80s authentication after %.80s",
 		 at->name,t);
-	mm_log (tmp,NIL);
+	mm_log (pwd,NIL);
 	fs_give ((void **) &t);
       }
       trial = 0;		/* initial trial count */
-      tmp[0] = '\0';		/* empty buffer */
-      if (LOCAL->netstream) do {
-	if (tmp[0]) mm_log (tmp,WARN);
-	if (pop3_send (stream,"AUTH",at->name) &&
-	    (*at->client) (pop3_challenge,pop3_response,mb,stream,&trial,usr)&&
-	    LOCAL->response) {
-	  if (*LOCAL->response == '+') return LONGT;
-	  if (!trial) {		/* if main program requested cancellation */
-	    mm_log ("POP3 Authentication cancelled",ERROR);
-	    return NIL;
-	  }
+      do {
+	if (t) {
+	  sprintf (pwd,"Retrying %s authentication after %s",at->name,t);
+	  mm_log (pwd,WARN);
+	  fs_give ((void **) &t);
 	}
-	t = cpystr (LOCAL->reply);
-	sprintf (tmp,"Retrying %s authentication after %s",at->name,t);
-      } while (LOCAL->netstream && trial && (trial < pop3_maxlogintrials));
+	if (pop3_send (stream,"AUTH",at->name) &&
+	    (*at->client) (pop3_challenge,pop3_response,"pop",mb,stream,&trial,
+			   usr) && LOCAL->response) {
+	  if (*LOCAL->response == '+') ret = LONGT;
+				/* if main program requested cancellation */
+	  else if (!trial) mm_log ("POP3 Authentication cancelled",ERROR);
+	}
+				/* remember response if error and no cancel */
+	if (!ret && trial) t = cpystr (LOCAL->reply);
+      } while (!ret && trial && (trial < pop3_maxlogintrials) &&
+	       LOCAL->netstream);
     }
     if (t) {			/* previous authenticator failed? */
-      sprintf (tmp,"Can not authenticate to POP3 server: %.80s",t);
-      mm_log (tmp,ERROR);
+      sprintf (pwd,"Can not authenticate to POP3 server: %.80s",t);
+      mm_log (pwd,ERROR);
       fs_give ((void **) &t);
     }
   }
+
   else if (stream->secure)
     mm_log ("Can't do secure authentication with this server",ERROR);
   else if (mb->authuser[0])
     mm_log ("Can't do /authuser with this server",ERROR);
+  else if (!LOCAL->cap.user) mm_log ("Can't login to this server",ERROR);
   else {			/* traditional login */
-    for (i = 0; LOCAL->netstream && (i < pop3_maxlogintrials); ++i) {
-      tmp[0] = '\0';		/* prompt user for password */
-      mm_login (mb,usr,tmp,i);
-      if (tmp[0]) {		/* send login sequence */
-	if (pop3_send (stream,"USER",usr) && pop3_send (stream,"PASS",tmp))
-	  return LONGT;		/* success */
+    trial = 0;			/* initial trial count */
+    do {
+      pwd[0] = 0;		/* prompt user for password */
+      mm_login (mb,usr,pwd,trial++);
+				/* user refused to give a password */
+      if (!pwd[0]) mm_log ("Login aborted",ERROR);
+				/* send login sequence */
+      else if (pop3_send (stream,"USER",usr) && pop3_send (stream,"PASS",pwd))
+	ret = LONGT;		/* success */
+      else {
 	mm_log (LOCAL->reply,WARN);
+	if (trial == pop3_maxlogintrials)
+	  mm_log ("Too many login failures",ERROR);
       }
-      else {			/* user refused to give a password */
-	mm_log ("Login aborted",ERROR);
-	return NIL;
-      }
-    }
-    mm_log ("Too many login failures",ERROR);
+    } while (!ret && pwd[0] && (trial < pop3_maxlogintrials) &&
+	     LOCAL->netstream);
   }
-  return NIL;			/* ran out of authenticators */
+  memset (pwd,0,MAILTMPLEN);	/* erase password */
+				/* get capabilities if logged in */
+  if (ret && capaok) pop3_capa (stream,flags);
+  return ret;
 }
 
 /* Get challenge to authenticator in binary
@@ -504,8 +618,7 @@ long pop3_response (void *s,char *response,unsigned long size)
   }
 				/* abort requested */
   else ret = net_sout (LOCAL->netstream,"*\015\012",3);
-				/* get response */
-  if (!pop3_reply (stream)) ret = NIL;
+  pop3_reply (stream);		/* get response */
   return ret;
 }
 
@@ -527,6 +640,9 @@ void pop3_close (MAILSTREAM *stream,long options)
     }
 				/* close POP3 connection */
     if (LOCAL->netstream) net_close (LOCAL->netstream);
+				/* clean up */
+    if (LOCAL->cap.implementation)
+      fs_give ((void **) &LOCAL->cap.implementation);
     if (LOCAL->txt) fclose (LOCAL->txt);
     LOCAL->txt = NIL;
     if (LOCAL->response) fs_give ((void **) &LOCAL->response);
