@@ -10,10 +10,10 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	22 May 1990
- * Last Edited:	17 November 2000
+ * Last Edited:	11 April 2001
  * 
  * The IMAP toolkit provided in this Distribution is
- * Copyright 2000 University of Washington.
+ * Copyright 2001 University of Washington.
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this Distribution.
  */
@@ -40,7 +40,7 @@ extern int errno;		/* just in case */
 DRIVER mtxdriver = {
   "mtx",			/* driver name */
 				/* driver flags */
-  DR_LOCAL|DR_MAIL|DR_CRLF|DR_NOSTICKY,
+  DR_LOCAL|DR_MAIL|DR_CRLF|DR_NOSTICKY|DR_LOCKING,
   (DRIVER *) NIL,		/* next driver */
   mtx_valid,			/* mailbox is valid for us */
   mtx_parameters,		/* manipulate parameters */
@@ -213,19 +213,19 @@ long mtx_rename (MAILSTREAM *stream,char *old,char *newname)
   struct stat sbuf;
   if (fd < 0) {			/* open mailbox */
     sprintf (tmp,"Can't open mailbox %.80s: %s",old,strerror (errno));
-    mm_log (tmp,ERROR);
+    MM_LOG (tmp,ERROR);
     return NIL;
   }
 				/* get exclusive parse/append permission */
   if ((ld = lockfd (fd,lock,LOCK_EX)) < 0) {
-    mm_log ("Unable to lock rename mailbox",ERROR);
+    MM_LOG ("Unable to lock rename mailbox",ERROR);
     return NIL;
   }
 				/* lock out other users */
   if (flock (fd,LOCK_EX|LOCK_NB)) {
     close (fd);			/* couldn't lock, give up on it then */
     sprintf (tmp,"Mailbox %.80s is in use by another process",old);
-    mm_log (tmp,ERROR);
+    MM_LOG (tmp,ERROR);
     unlockfd (ld,lock);		/* release exclusive parse/append permission */
     return NIL;
   }
@@ -234,7 +234,7 @@ long mtx_rename (MAILSTREAM *stream,char *old,char *newname)
     if (!((s = mtx_file (tmp,newname)) && *s)) {
       sprintf (tmp,"Can't rename mailbox %.80s to %.80s: invalid name",
 	       old,newname);
-      mm_log (tmp,ERROR);
+      MM_LOG (tmp,ERROR);
       ret = NIL;		/* set failure */
     }
     if (s = strrchr (s,'/')) {	/* found superior to destination name? */
@@ -242,28 +242,28 @@ long mtx_rename (MAILSTREAM *stream,char *old,char *newname)
       *s = '\0';		/* tie off to get just superior */
 				/* name doesn't exist, create it */
       if ((stat (tmp,&sbuf) || ((sbuf.st_mode & S_IFMT) != S_IFDIR)) &&
-	  !dummy_create (stream,tmp)) ret = NIL;
+	  !dummy_create_path (stream,tmp,get_dir_protection (newname)))
+	ret = NIL;
       else *s = c;		/* restore full name */
     }
 				/* rename the file */
     if (ret && rename (file,tmp)) {
       sprintf (tmp,"Can't rename mailbox %.80s to %.80s: %s",old,newname,
 	       strerror (errno));
-      mm_log (tmp,ERROR);
+      MM_LOG (tmp,ERROR);
       ret = NIL;		/* set failure */
     }
   }
   else if (unlink (file)) {
     sprintf (tmp,"Can't delete mailbox %.80s: %s",old,strerror (errno));
-    mm_log (tmp,ERROR);
+    MM_LOG (tmp,ERROR);
     ret = NIL;			/* set failure */
   }
   flock (fd,LOCK_UN);		/* release lock on the file */
   close (fd);			/* close the file */
   unlockfd (ld,lock);		/* release exclusive parse/append permission */
 				/* recreate file if renamed INBOX */
-  if (ret && !strcmp (ucase (strcpy (tmp,old)),"INBOX"))
-    dummy_create (NIL,"INBOX.MTX");
+  if (ret && !compare_cstring (old,"INBOX")) dummy_create (NIL,"INBOX.MTX");
   return ret;			/* return success */
 }
 
@@ -302,8 +302,7 @@ long mtx_status (MAILSTREAM *stream,char *mbx,long flags)
 				/* kludge but probably good enough */
     status.uidnext += systream->nmsgs;
   }
-				/* pass status to main program */
-  mm_status (stream,mbx,&status);
+  MM_STATUS(stream,mbx,&status);/* pass status to main program */
   if (tstream) mail_close (tstream);
   if (systream) mail_close (systream);
   return T;			/* success */
@@ -327,11 +326,11 @@ MAILSTREAM *mtx_open (MAILSTREAM *stream)
       (fd = open (mtx_file (tmp,stream->mailbox),O_RDWR,NIL)) < 0) {
     if ((fd = open (mtx_file (tmp,stream->mailbox),O_RDONLY,NIL)) < 0) {
       sprintf (tmp,"Can't open mailbox: %.80s",strerror (errno));
-      mm_log (tmp,ERROR);
+      MM_LOG (tmp,ERROR);
       return NIL;
     }
     else if (!stream->rdonly) {	/* got it, but readonly */
-      mm_log ("Can't get write access to mailbox, access is readonly",WARN);
+      MM_LOG ("Can't get write access to mailbox, access is readonly",WARN);
       stream->rdonly = T;
     }
   }
@@ -340,12 +339,12 @@ MAILSTREAM *mtx_open (MAILSTREAM *stream)
   LOCAL->buf = (char *) fs_get (MAXMESSAGESIZE + 1);
   LOCAL->buflen = MAXMESSAGESIZE;
 				/* note if an INBOX or not */
-  stream->inbox = !strcmp(ucase (strcpy (LOCAL->buf,stream->mailbox)),"INBOX");
+  stream->inbox = !compare_cstring (stream->mailbox,"INBOX");
   fs_give ((void **) &stream->mailbox);
   stream->mailbox = cpystr (tmp);
 				/* get shared parse permission */
   if ((ld = lockfd (fd,tmp,LOCK_SH)) < 0) {
-    mm_log ("Unable to lock open mailbox",ERROR);
+    MM_LOG ("Unable to lock open mailbox",ERROR);
     return NIL;
   }
   (*bn) (BLOCK_FILELOCK,NIL);
@@ -360,7 +359,7 @@ MAILSTREAM *mtx_open (MAILSTREAM *stream)
 				/* parse mailbox */
   stream->nmsgs = stream->recent = 0;
   if (mtx_ping (stream) && !stream->nmsgs)
-    mm_log ("Mailbox is empty",(long) NIL);
+    MM_LOG ("Mailbox is empty",(long) NIL);
   if (!LOCAL) return NIL;	/* failure if stream died */
   stream->perm_seen = stream->perm_deleted =
     stream->perm_flagged = stream->perm_answered = stream->perm_draft =
@@ -455,7 +454,7 @@ long mtx_text (MAILSTREAM *stream,unsigned long msgno,STRING *bs,long flags)
     elt->seen = T;		/* mark message as seen */
 				/* recalculate status */
     mtx_update_status (stream,msgno,T);
-    mm_flags (stream,msgno);
+    MM_FLAGS (stream,msgno);
   }
 				/* find header position */
   i = mtx_hdrpos (stream,msgno,&j);
@@ -528,7 +527,7 @@ long mtx_ping (MAILSTREAM *stream)
 				/* check for changed message status */
     if (LOCAL->mustcheck || LOCAL->shouldcheck) {
       if (LOCAL->shouldcheck)	/* babble when we do this unilaterally */
-	mm_notify (stream,"[CHECK] Checking for flag updates",NIL);
+	MM_NOTIFY (stream,"[CHECK] Checking for flag updates",NIL);
       while (i <= stream->nmsgs) mtx_elt (stream,i++);
       LOCAL->mustcheck = LOCAL->shouldcheck = NIL;
     }
@@ -571,7 +570,7 @@ void mtx_check (MAILSTREAM *stream)
 {
 				/* mark that a check is desired */
   if (LOCAL) LOCAL->mustcheck = T;
-  if (mtx_ping (stream)) mm_log ("Check completed",(long) NIL);
+  if (mtx_ping (stream)) MM_LOG ("Check completed",(long) NIL);
 }
 
 /* MTX mail snarf messages from system inbox
@@ -591,7 +590,7 @@ void mtx_snarf (MAILSTREAM *stream)
   if ((time (0) < (LOCAL->lastsnarf + 30)) ||
       (!strcmp (sysinbox (),stream->mailbox)) ||
       ((ld = lockfd (LOCAL->fd,lock,LOCK_EX)) < 0)) return;
-  mm_critical (stream);		/* go critical */
+  MM_CRITICAL (stream);		/* go critical */
 				/* see if anything there */
   if (!stat (sysinbox (),&sbuf) && sbuf.st_size) {
     fstat (LOCAL->fd,&sbuf);	/* yes, get current file size */
@@ -631,7 +630,7 @@ void mtx_snarf (MAILSTREAM *stream)
       }
       else {
 	sprintf (LOCAL->buf,"Can't copy new mail: %s",strerror (errno));
-	mm_log (LOCAL->buf,ERROR);
+	MM_LOG (LOCAL->buf,ERROR);
 	ftruncate (LOCAL->fd,sbuf.st_size);
       }
       fstat (LOCAL->fd,&sbuf);	/* yes, get current file size */
@@ -639,7 +638,7 @@ void mtx_snarf (MAILSTREAM *stream)
     }
     if (sysibx) mail_close (sysibx);
   }
-  mm_nocritical (stream);	/* release critical */
+  MM_NOCRITICAL (stream);	/* release critical */
   unlockfd (ld,lock);		/* release exclusive parse/append permission */
   LOCAL->lastsnarf = time (0);	/* note time of last snarf */
 }
@@ -663,7 +662,7 @@ void mtx_expunge (MAILSTREAM *stream)
 				/* do nothing if stream dead */
   if (!mtx_ping (stream)) return;
   if (stream->rdonly) {		/* won't do on readonly files! */
-    mm_log ("Expunge ignored on readonly mailbox",WARN);
+    MM_LOG ("Expunge ignored on readonly mailbox",WARN);
     return;
   }
   if (LOCAL->filetime && !LOCAL->shouldcheck) {
@@ -681,7 +680,7 @@ void mtx_expunge (MAILSTREAM *stream)
    */
 				/* get exclusive parse/append permission */
   if ((ld = lockfd (LOCAL->fd,lock,LOCK_EX)) < 0) {
-    mm_log ("Unable to lock expunge mailbox",ERROR);
+    MM_LOG ("Unable to lock expunge mailbox",ERROR);
     return;
   }
 				/* get exclusive access */
@@ -689,12 +688,12 @@ void mtx_expunge (MAILSTREAM *stream)
     (*bn) (BLOCK_FILELOCK,NIL);
     flock (LOCAL->fd,LOCK_SH);	/* recover previous lock */
     (*bn) (BLOCK_NONE,NIL);
-    mm_log("Can't expunge because mailbox is in use by another process",ERROR);
+    MM_LOG("Can't expunge because mailbox is in use by another process",ERROR);
     unlockfd (ld,lock);		/* release exclusive parse/append permission */
     return;
   }
 
-  mm_critical (stream);		/* go critical */
+  MM_CRITICAL (stream);	/* go critical */
   recent = stream->recent;	/* get recent now that pinged and locked */
   while (i <= stream->nmsgs) {	/* for each message */
     elt = mtx_elt (stream,i);	/* get cache element */
@@ -718,8 +717,8 @@ void mtx_expunge (MAILSTREAM *stream)
 	while (T) {
 	  lseek (LOCAL->fd,pos,L_SET);
 	  if (write (LOCAL->fd,LOCAL->buf,m) > 0) break;
-	  mm_notify (stream,strerror (errno),WARN);
-	  mm_diskerror (stream,errno,T);
+	  MM_NOTIFY (stream,strerror (errno),WARN);
+	  MM_DISKERROR (stream,errno,T);
 	}
 	pos += m;		/* new position */
 	j += m;			/* next chunk, perhaps */
@@ -734,19 +733,19 @@ void mtx_expunge (MAILSTREAM *stream)
     if (pos != (LOCAL->filesize -= delta)) {
       sprintf (LOCAL->buf,"Calculated size mismatch %lu != %lu, delta = %lu",
 	       (unsigned long) pos,(unsigned long) LOCAL->filesize,delta);
-      mm_log (LOCAL->buf,WARN);
+      MM_LOG (LOCAL->buf,WARN);
       LOCAL->filesize = pos;	/* fix it then */
     }
     ftruncate (LOCAL->fd,LOCAL->filesize);
     sprintf (LOCAL->buf,"Expunged %lu messages",n);
 				/* output the news */
-    mm_log (LOCAL->buf,(long) NIL);
+    MM_LOG (LOCAL->buf,(long) NIL);
   }
-  else mm_log ("No messages deleted, so no update needed",(long) NIL);
+  else MM_LOG ("No messages deleted, so no update needed",(long) NIL);
   fsync (LOCAL->fd);		/* force disk update */
   fstat (LOCAL->fd,&sbuf);	/* get new write time */
   LOCAL->filetime = sbuf.st_mtime;
-  mm_nocritical (stream);	/* release critical */
+  MM_NOCRITICAL (stream);	/* release critical */
 				/* notify upper level of new mailbox size */
   mail_exists (stream,stream->nmsgs);
   mail_recent (stream,recent);
@@ -778,19 +777,19 @@ long mtx_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
 				/* make sure valid mailbox */
   if (!mtx_isvalid (mailbox,LOCAL->buf)) switch (errno) {
   case ENOENT:			/* no such file? */
-    mm_notify (stream,"[TRYCREATE] Must create mailbox before copy",NIL);
+    MM_NOTIFY (stream,"[TRYCREATE] Must create mailbox before copy",NIL);
     return NIL;
   case 0:			/* merely empty file? */
     break;
   case EINVAL:
     if (pc) return (*pc) (stream,sequence,mailbox,options);
     sprintf (LOCAL->buf,"Invalid MTX-format mailbox name: %.80s",mailbox);
-    mm_log (LOCAL->buf,ERROR);
+    MM_LOG (LOCAL->buf,ERROR);
     return NIL;
   default:
     if (pc) return (*pc) (stream,sequence,mailbox,options);
     sprintf (LOCAL->buf,"Not a MTX-format mailbox: %.80s",mailbox);
-    mm_log (LOCAL->buf,ERROR);
+    MM_LOG (LOCAL->buf,ERROR);
     return NIL;
   }
   if (!((options & CP_UID) ? mail_uid_sequence (stream,sequence) :
@@ -798,13 +797,13 @@ long mtx_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
 				/* got file? */  
   if ((fd=  open (mtx_file (file,mailbox),O_RDWR|O_CREAT,S_IREAD|S_IWRITE))<0){
     sprintf (LOCAL->buf,"Unable to open copy mailbox: %s",strerror (errno));
-    mm_log (LOCAL->buf,ERROR);
+    MM_LOG (LOCAL->buf,ERROR);
     return NIL;
   }
-  mm_critical (stream);		/* go critical */
+  MM_CRITICAL (stream);		/* go critical */
 				/* get exclusive parse/append permission */
   if ((ld = lockfd (fd,lock,LOCK_EX)) < 0) {
-    mm_log ("Unable to lock copy mailbox",ERROR);
+    MM_LOG ("Unable to lock copy mailbox",ERROR);
     return NIL;
   }
   fstat (fd,&sbuf);		/* get current file size */
@@ -825,7 +824,7 @@ long mtx_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
 				/* make sure all the updates take */
   if (!(ret && (ret = !fsync (fd)))) {
     sprintf (LOCAL->buf,"Unable to write message: %s",strerror (errno));
-    mm_log (LOCAL->buf,ERROR);
+    MM_LOG (LOCAL->buf,ERROR);
     ftruncate (fd,sbuf.st_size);
   }
   tp[0] = sbuf.st_atime;	/* preserve atime and mtime */
@@ -833,7 +832,7 @@ long mtx_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
   utime (file,tp);		/* set the times */
   close (fd);			/* close the file */
   unlockfd (ld,lock);		/* release exclusive parse/append permission */
-  mm_nocritical (stream);	/* release critical */
+  MM_NOCRITICAL (stream);	/* release critical */
 				/* delete all requested messages */
   if (ret && (options & CP_MOVE)) {
     for (i = 1; i <= stream->nmsgs; i++)
@@ -883,7 +882,7 @@ long mtx_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
 	((mailbox[4] == 'X') || (mailbox[4] == 'x')) && !mailbox[5])
       dummy_create (NIL,"INBOX.MTX");
     else {
-      mm_notify (stream,"[TRYCREATE] Must create mailbox before append",NIL);
+      MM_NOTIFY (stream,"[TRYCREATE] Must create mailbox before append",NIL);
       return NIL;
     }
 				/* falls through */
@@ -891,34 +890,34 @@ long mtx_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
     break;
   case EINVAL:
     sprintf (tmp,"Invalid MTX-format mailbox name: %.80s",mailbox);
-    mm_log (tmp,ERROR);
+    MM_LOG (tmp,ERROR);
     return NIL;
   default:
     sprintf (tmp,"Not a MTX-format mailbox: %.80s",mailbox);
-    mm_log (tmp,ERROR);
+    MM_LOG (tmp,ERROR);
     return NIL;
   }
 				/* get first message */
-  if (!(*af) (stream,data,&flags,&date,&message)) return NIL;
+  if (!MM_APPEND (af) (stream,data,&flags,&date,&message)) return NIL;
 
 				/* open destination mailbox */
   if (((fd = open (mtx_file (file,mailbox),O_WRONLY|O_APPEND|O_CREAT,
 		   S_IREAD|S_IWRITE)) < 0) || !(df = fdopen (fd,"ab"))) {
     sprintf (tmp,"Can't open append mailbox: %s",strerror (errno));
-    mm_log (tmp,ERROR);
+    MM_LOG (tmp,ERROR);
     return NIL;
   }
 				/* get parse/append permission */
   if ((ld = lockfd (fd,lock,LOCK_EX)) < 0) {
-    mm_log ("Unable to lock append mailbox",ERROR);
+    MM_LOG ("Unable to lock append mailbox",ERROR);
     close (fd);
     return NIL;
   }
-  mm_critical (stream);		/* go critical */
+  MM_CRITICAL (stream);		/* go critical */
   fstat (fd,&sbuf);		/* get current file size */
   do {				/* parse flags */
     if (!SIZE (message)) {	/* guard against zero-length */
-      mm_log ("Append of zero-length message",ERROR);
+      MM_LOG ("Append of zero-length message",ERROR);
       ret = NIL;
       break;
     }
@@ -928,7 +927,7 @@ long mtx_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
     if (date) {			/* parse date if given */
       if (!mail_parse_date (&elt,date)) {
 	sprintf (tmp,"Bad date in append: %.80s",date);
-	mm_log (tmp,ERROR);
+	MM_LOG (tmp,ERROR);
 	ret = NIL;		/* mark failure */
 	break;
       }
@@ -942,7 +941,7 @@ long mtx_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
       if (i) do c = 0xff & SNX (message);
       while ((putc (c,df) != EOF) && --i);
 				/* get next message */
-      if (i || !(*af) (stream,data,&flags,&date,&message)) ret = NIL;
+      if (i || !MM_APPEND (af) (stream,data,&flags,&date,&message)) ret = NIL;
     }
   } while (ret && message);
 				/* if error... */
@@ -950,7 +949,7 @@ long mtx_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
     ftruncate (fd,sbuf.st_size);/* revert file */
     close (fd);			/* make sure fclose() doesn't corrupt us */
     sprintf (tmp,"Message append failed: %s",strerror (errno));
-    mm_log (tmp,ERROR);
+    MM_LOG (tmp,ERROR);
     ret = NIL;
   }
   tp[0] = sbuf.st_atime;	/* preserve atime and mtime */
@@ -958,7 +957,7 @@ long mtx_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
   utime (file,tp);		/* set the times */
   fclose (df);			/* close the file */
   unlockfd (ld,lock);		/* release exclusive parse/append permission */
-  mm_nocritical (stream);	/* release critical */
+  MM_NOCRITICAL (stream);	/* release critical */
   return ret;
 }
 
@@ -1001,7 +1000,7 @@ long mtx_parse (MAILSTREAM *stream)
   if (sbuf.st_size < curpos) {	/* sanity check */
     sprintf (tmp,"Mailbox shrank from %lu to %lu!",
 	     (unsigned long) curpos,(unsigned long) sbuf.st_size);
-    mm_log (tmp,ERROR);
+    MM_LOG (tmp,ERROR);
     mtx_close (stream,NIL);
     return NIL;
   }
@@ -1013,7 +1012,7 @@ long mtx_parse (MAILSTREAM *stream)
       sprintf (tmp,"Unable to read internal header at %lu, size = %lu: %s",
 	       (unsigned long) curpos,(unsigned long) sbuf.st_size,
 	       i ? strerror (errno) : "no data read");
-      mm_log (tmp,ERROR);
+      MM_LOG (tmp,ERROR);
       mtx_close (stream,NIL);
       return NIL;
     }
@@ -1021,7 +1020,7 @@ long mtx_parse (MAILSTREAM *stream)
     if (!((s = strchr (LOCAL->buf,'\015')) && (s[1] == '\012'))) {
       sprintf (tmp,"Unable to find CRLF at %lu in %lu bytes, text: %s",
 	       (unsigned long) curpos,i,LOCAL->buf);
-      mm_log (tmp,ERROR);
+      MM_LOG (tmp,ERROR);
       mtx_close (stream,NIL);
       return NIL;
     }
@@ -1030,7 +1029,7 @@ long mtx_parse (MAILSTREAM *stream)
     if (!((s = strchr (LOCAL->buf,',')) && (t = strchr (s+1,';')))) {
       sprintf (tmp,"Unable to parse internal header at %lu: %s",
 	       (unsigned long) curpos,LOCAL->buf);
-      mm_log (tmp,ERROR);
+      MM_LOG (tmp,ERROR);
       mtx_close (stream,NIL);
       return NIL;
     }
@@ -1061,7 +1060,7 @@ long mtx_parse (MAILSTREAM *stream)
     else {			/* oops */
       sprintf (tmp,"Unable to parse internal header elements at %ld: %s,%s;%s",
 	       curpos,LOCAL->buf,x,t);
-      mm_log (tmp,ERROR);
+      MM_LOG (tmp,ERROR);
       mtx_close (stream,NIL);
       return NIL;
     }
@@ -1070,7 +1069,7 @@ long mtx_parse (MAILSTREAM *stream)
       sprintf (tmp,"Last message (at %lu) runs past end of file (%lu > %lu)",
 	       elt->private.special.offset,(unsigned long) curpos,
 	       (unsigned long) sbuf.st_size);
-      mm_log (tmp,ERROR);
+      MM_LOG (tmp,ERROR);
       mtx_close (stream,NIL);
       return NIL;
     }
@@ -1129,7 +1128,7 @@ MESSAGECACHE *mtx_elt (MAILSTREAM *stream,unsigned long msgno)
   if ((old.seen != elt->seen) || (old.deleted != elt->deleted) ||
       (old.flagged != elt->flagged) || (old.answered != elt->answered) ||
       (old.draft != elt->draft) || (old.user_flags != elt->user_flags))
-    mm_flags (stream,msgno);	/* let top level know */
+    MM_FLAGS (stream,msgno);	/* let top level know */
   return elt;
 }
 
