@@ -10,15 +10,15 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	24 May 1993
- * Last Edited:	19 November 2000
+ * Last Edited:	9 April 2001
  * 
  * The IMAP toolkit provided in this Distribution is
- * Copyright 2000 University of Washington.
+ * Copyright 2001 University of Washington.
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this Distribution.
  */
 
-/* Thanks to Nicholas Sheppard for the original version*/
+/* Thanks to Nicholas Sheppard for the original version */
 
 
 #include <ctype.h>
@@ -56,7 +56,7 @@ DRIVER dummydriver = {
   dummy_create,			/* create mailbox */
   dummy_delete,			/* delete mailbox */
   dummy_rename,			/* rename mailbox */
-  NIL,				/* status of mailbox */
+  mail_status_default,		/* status of mailbox */
   dummy_open,			/* open mailbox */
   dummy_close,			/* close mailbox */
   NIL,				/* fetch message "fast" attributes */
@@ -345,7 +345,7 @@ long dummy_listed (MAILSTREAM *stream,char delimiter,char *name,
 long dummy_create (MAILSTREAM *stream,char *mailbox)
 {
   char tmp[MAILTMPLEN];
-  if (strcmp (ucase (strcpy (tmp,mailbox)),"INBOX") && dummy_file(tmp,mailbox))
+  if (compare_cstring (mailbox,"INBOX") && dummy_file (tmp,mailbox))
     return dummy_create_path (stream,tmp,NIL);
   sprintf (tmp,"Can't create %s: invalid name",mailbox);
   mm_log (tmp,ERROR);
@@ -448,7 +448,7 @@ long dummy_rename (MAILSTREAM *stream,char *old,char *newname)
     s[1] = c;			/* restore character after delimiter */
   }
 				/* rename of non-ex INBOX creates dest */
-  if (!strcmp (ucase (strcpy (tmp,old)),"INBOX") &&
+  if (!compare_cstring (old,"INBOX") &&
       stat (dummy_file (tmp,old),&sbuf)) return dummy_create (NIL,mbx);
   if (rename (dummy_file (tmp,old),mbx)) {
     sprintf (tmp,"Can't rename mailbox %s to %s: %s",old,newname,
@@ -475,7 +475,7 @@ MAILSTREAM *dummy_open (MAILSTREAM *stream)
 				/* can we open the file? */
   if ((fd = open (dummy_file (tmp,stream->mailbox),O_RDONLY,NIL)) < 0) {
 				/* no, error unless INBOX */
-    if (strcmp (ucase (strcpy (tmp,stream->mailbox)),"INBOX"))
+    if (compare_cstring (stream->mailbox,"INBOX"))
       sprintf (err,"%s: %s",strerror (errno),stream->mailbox);
   }
   else {			/* file had better be empty then */
@@ -588,8 +588,8 @@ long dummy_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
   int e;
   char tmp[MAILTMPLEN];
   MAILSTREAM *ts = default_proto (T);
-  if ((strcmp (ucase (strcpy (tmp,mailbox)),"INBOX")) &&
-	   ((fd = open (dummy_file (tmp,mailbox),O_RDONLY,NIL)) < 0)) {
+  if (compare_cstring (mailbox,"INBOX") &&
+      ((fd = open (dummy_file (tmp,mailbox),O_RDONLY,NIL)) < 0)) {
     if ((e = errno) == ENOENT)	/* failed, was it no such file? */
       mm_notify (stream,"[TRYCREATE] Must create mailbox before append",
 		 (long) NIL);
