@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: basic.c,v 4.30 2000/11/09 21:53:43 hubert Exp $";
+static char rcsid[] = "$Id: basic.c,v 4.31 2001/06/21 17:49:51 hubert Exp $";
 #endif
 /*
  * Program:	Cursor manipulation functions
@@ -21,7 +21,7 @@ static char rcsid[] = "$Id: basic.c,v 4.30 2000/11/09 21:53:43 hubert Exp $";
  * permission of the University of Washington.
  * 
  * Pine, Pico, and Pilot software and its included text are Copyright
- * 1989-2000 by the University of Washington.
+ * 1989-2001 by the University of Washington.
  * 
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this distribution.
@@ -226,7 +226,6 @@ backline(f, n)
 int f, n;
 {
     register LINE   *dlp;
-    register int    status = 0;
 
     if (n < 0)
       return (forwline(f, -n));
@@ -524,7 +523,6 @@ int		movedot;
 {
     register LINE   *lp, *tp;
     register int    nl;
-    int             status = 0;
     int		    i;
 
     if(Pmaster && Pmaster->headents){
@@ -712,9 +710,9 @@ int f, n;
 	   ToggleHeader (0);
         }
     }
+#endif
 
     return (TRUE);
-#endif
 }
 
 
