@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: browse.c,v 4.84 1999/12/22 17:56:55 hubert Exp $";
+static char rcsid[] = "$Id: browse.c,v 4.85 2001/06/21 17:49:51 hubert Exp $";
 #endif
 /*
  * Program:	Routines to support file browser in pico and Pine composer
@@ -21,7 +21,7 @@ static char rcsid[] = "$Id: browse.c,v 4.84 1999/12/22 17:56:55 hubert Exp $";
  * permission of the University of Washington.
  * 
  * Pine, Pico, and Pilot software and its included text are Copyright
- * 1989-1999 by the University of Washington.
+ * 1989-2001 by the University of Washington.
  * 
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this distribution.
@@ -95,15 +95,15 @@ static	char	*browser_title = NULL;
     struct bmaster *getfcells(char *);
     int    PaintCell(int, int, int, struct fcell *, int);
     int    PaintBrowser(struct bmaster *, int, int *, int *);
-    int    BrowserKeys(void);
-    int    layoutcells(struct bmaster *);
-    int    percdircells(struct bmaster *);
+    void   BrowserKeys(void);
+    void   layoutcells(struct bmaster *);
+    void   percdircells(struct bmaster *);
     int    PlaceCell(struct bmaster *, struct fcell *, int *, int *);
-    int    zotfcells(struct fcell *);
-    int    zotmaster(struct bmaster **);
+    void   zotfcells(struct fcell *);
+    void   zotmaster(struct bmaster **);
     struct fcell *FindCell(struct bmaster *, char *);
     int    sisin(char *, char *);
-    int    BrowserAnchor(char *);
+    void   BrowserAnchor(char *);
     void   ClearBrowserScreen(void);
     void   BrowserRunChild(char *, char *);
     int    LikelyASCII(char *);
@@ -111,14 +111,15 @@ static	char	*browser_title = NULL;
     struct bmaster *getfcells();
     int    PaintCell();
     int    PaintBrowser();
-    int    BrowserKeys();
-    int    percdircells();
+    void   BrowserKeys();
+    void   layoutcells();
+    void   percdircells();
     int    PlaceCell();
-    int    zotfcells();
-    int    zotmaster();
+    void   zotfcells();
+    void   zotmaster();
     struct fcell *FindCell();
     int    sisin();
-    int    BrowserAnchor();
+    void   BrowserAnchor();
     void   ClearBrowserScreen();
     void   BrowserRunChild();
     int    LikelyASCII();
@@ -1846,6 +1847,7 @@ int *row, *col;
 /*
  * BrowserKeys - just paints the keyhelp at the bottom of the display
  */
+void
 BrowserKeys()
 {
     menu_browse[QUIT_KEY].name  = (gmode&MDBRONLY) ? "Q" : "E";
@@ -1875,6 +1877,7 @@ BrowserKeys()
  * layoutcells - figure out max length of cell and how many cells can 
  *               go on a line of the display
  */
+void
 layoutcells(mp)
 struct bmaster *mp;
 {
@@ -1900,6 +1903,7 @@ struct bmaster *mp;
  * percdircells - bubble all the directory cells to the top of the
  *                list.
  */
+void
 percdircells(mp)
 struct bmaster *mp;
 {
@@ -2003,6 +2007,7 @@ int    *x, *y;
 /*
  * zotfcells - clean up malloc'd cells of file names
  */
+void
 zotfcells(hp)
 struct fcell *hp;
 {
@@ -2020,6 +2025,7 @@ struct fcell *hp;
 /*
  * zotmaster - blast the browser master struct
  */
+void
 zotmaster(mp)
 struct bmaster **mp;
 {
@@ -2088,6 +2094,7 @@ char *s1, *s2;
 /*
  * set_browser_title - 
  */
+void
 set_browser_title(s)
 char *s;
 {
@@ -2098,6 +2105,7 @@ char *s;
 /*
  * BrowserAnchor - draw the browser's anchor line.
  */
+void
 BrowserAnchor(dir)
 char *dir;
 {
