@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: display.c,v 4.100 2000/11/09 21:53:43 hubert Exp $";
+static char rcsid[] = "$Id: display.c,v 4.102 2001/06/21 17:49:51 hubert Exp $";
 #endif
 /*
  * Program:	Display functions
@@ -21,7 +21,7 @@ static char rcsid[] = "$Id: display.c,v 4.100 2000/11/09 21:53:43 hubert Exp $";
  * permission of the University of Washington.
  * 
  * Pine, Pico, and Pilot software and its included text are Copyright
- * 1989-2000 by the University of Washington.
+ * 1989-2001 by the University of Washington.
  * 
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this distribution.
@@ -981,8 +981,8 @@ WINDOW *wp;
 	register char *cp;
 	register int n;		/* cursor position count */
 	register BUFFER *bp;
-	register i;		/* loop index */
-	register lchar;		/* character to draw line in buffer with */
+	register int i;		/* loop index */
+	register int lchar;	/* character to draw line in buffer with */
 	char     tline[NLINE];	/* buffer for part of mode line */
 	CELL     c;
 
@@ -2195,11 +2195,18 @@ int   a;				/* and its attribute */
 {
     if((ttcol >= 0 && ttcol < term.t_ncol) 
        && (ttrow >= 0 && ttrow <= term.t_nrow)){
-/*	(*term.t_rev)(a);*/
-	(*term.t_putchar)(c);			/* write it */
-/*	(*term.t_rev)(!a);*/
-	pscreen[ttrow]->v_text[ttcol].c = c;	/* keep track of it */
-	pscreen[ttrow]->v_text[ttcol++].a = a;	/* keep track of it */
+       
+	/*
+	 * Some terminals scroll when you write in the lower right corner
+	 * of the screen, so don't write there.
+	 */
+	if(!(ttrow == term.t_nrow && ttcol == term.t_ncol -1)){
+	    (*term.t_putchar)(c);			/* write it */
+	    pscreen[ttrow]->v_text[ttcol].c = c;	/* keep track of it */
+	    pscreen[ttrow]->v_text[ttcol].a = a;	/* keep track of it */
+	}
+
+	ttcol++;
     }
 }
 
