@@ -1,5 +1,5 @@
 /*
- * $Id: efunc.h,v 4.84 2000/12/28 23:20:48 hubert Exp $
+ * $Id: efunc.h,v 4.86 2001/07/13 19:39:27 hubert Exp $
  *
  * Program:	Pine's composer and pico's function declarations
  *
@@ -20,7 +20,7 @@
  * permission of the University of Washington.
  * 
  * Pine, Pico, and Pilot software and its included text are Copyright
- * 1989-2000 by the University of Washington.
+ * 1989-2001 by the University of Washington.
  * 
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this distribution.
@@ -84,7 +84,7 @@ extern	int bindtokey PROTO((int c, int (*) PROTO((int, int))));
 /* browse.c */
 extern	int FileBrowse PROTO((char *, int, char *, int, char *, int));
 extern	int ResizeBrowser PROTO((void));
-extern	int set_browser_title PROTO((char *));
+extern	void set_browser_title PROTO((char *));
 
 /* buffer.c */
 extern	int anycb PROTO((void));
@@ -205,7 +205,7 @@ extern	int ttputc PROTO((int));
 extern	int ttflush PROTO((void));
 extern	int ttgetc PROTO((int, int (*)(), void (*)()));
 extern	int simple_ttgetc PROTO((int (*)(), void (*)()));
-extern	int ttgetwinsz PROTO((int *, int *));
+extern	void ttgetwinsz PROTO((int *, int *));
 extern	int GetKey PROTO((void));
 extern	int alt_editor PROTO((int, int));
 extern	void picosigs PROTO((void));
@@ -221,7 +221,7 @@ extern	int homeless PROTO((char *));
 extern	char *errstr PROTO((int));
 extern	char *getfnames PROTO((char *, char *, int *, char *));
 extern	void fioperr PROTO((int, char *));
-extern	int fixpath PROTO((char *, size_t));
+extern	void fixpath PROTO((char *, size_t));
 extern	char *pfnexpand PROTO((char *, size_t));
 extern	int compresspath PROTO((char *, char *, int));
 extern	void tmpname PROTO((char *, char *));
@@ -250,6 +250,7 @@ extern	int (*pcollator)();
 extern  COLOR_PAIR *new_color_pair PROTO((char *, char *));
 extern	COLOR_PAIR *pico_get_cur_color PROTO((void));
 extern	COLOR_PAIR *pico_get_rev_color PROTO((void));
+extern	COLOR_PAIR *pico_get_normal_color PROTO((void));
 extern	void free_color_pair PROTO((COLOR_PAIR **));
 extern	void pico_nfcolor PROTO((char *));
 extern	void pico_nbcolor PROTO((char *));
@@ -305,11 +306,13 @@ extern	int backdel PROTO((int, int));
 extern	int killtext PROTO((int, int));
 extern	int yank PROTO((int, int));
 extern	char *colorx PROTO((int));
+extern	COLOR_PAIR *pico_apply_rev_color PROTO((COLOR_PAIR *, int));
 
 /* region.c */
 extern	int killregion PROTO((int, int));
 extern	int deleteregion PROTO((int, int));
 extern	int markregion PROTO((int));
+extern	void unmarkbuffer PROTO((void));
 
 /* search.c */
 extern	int forwsearch PROTO((int, int));
