@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: args.c,v 4.103 2000/11/09 22:26:47 hubert Exp $";
+static char rcsid[] = "$Id: args.c,v 4.107 2001/06/22 23:44:34 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -22,7 +22,7 @@ static char rcsid[] = "$Id: args.c,v 4.103 2000/11/09 22:26:47 hubert Exp $";
    permission of the University of Washington.
 
    Pine, Pico, and Pilot software and its included text are Copyright
-   1989-2000 by the University of Washington.
+   1989-2001 by the University of Washington.
 
    The full text of our legal notices is contained in the file called
    CPYRIGHT, included with this distribution.
@@ -137,6 +137,8 @@ char *args_pine_args[] = {
 "\t\tsettings for a particular platform, can be a local file or",
 "\t\ta remote folder.",
 " -v \t\tVersion - show version information",
+" -version\tVersion - show version information",
+" -supported\tList supported options",
 #if defined(OS2)
 " -w <rows>\tSet window size in rows on startup",
 #endif
@@ -145,13 +147,10 @@ char *args_pine_args[] = {
 "\t\tStandard input redirection is not allowed with URLs.",
 "\t\tFor mailto URLs, 'body='text should be used in place of",
 "\t\tinput redirection.",
-" -create_lu <abook_file> <ab_sort_type>   create .lu from script",
-" -copy_pinerc <local_pinerc> <remote_pinerc>   copy local pinerc to remote",
-" -copy_abook <local_abook> <remote_abook>   copy local addressbook to remote",
-#ifdef notdef
-" -nr\t\tSpecial mode for UWIN",
-" -a\t\tSpecial anonymous mode for UWIN",
-#endif /* notdef */
+" -create_lu <abook_file> <ab_sort_type>       create .lu from script",
+" -copy_pinerc <local_pinerc> <remote_pinerc>  copy local pinerc to remote",
+" -copy_abook <local_abook> <remote_abook>     copy local addressbook to remote",
+" -convert_sigs -p <pinerc>   convert signatures to literal signatures",
 #if	defined(_WINDOWS)
 " -registry <cmd>\tWhere cmd is set,clear,dump",
 #endif
@@ -198,7 +197,6 @@ pine_args(pine_state, argc, argv, args)
     char *lc		      = NULL;
     int   do_help             = 0;
     int   do_conf             = 0;
-    int   anonymous           = 0;
     int   usage               = 0;
     int   do_use_fk           = 0;
     int   do_can_suspend      = 0;
@@ -287,6 +285,14 @@ Loop: while(--ac > 0)
 
 		  goto Loop;
 	      }
+	      else if(strcmp(*av, "convert_sigs") == 0){
+		  ps_global->convert_sigs = 1;
+		  goto Loop;
+	      }
+	      else if(strcmp(*av, "supported") == 0){
+		  ps_global->dump_supported_options = 1;
+		  goto Loop;
+	      }
 	      else if(strcmp(*av, "copy_pinerc") == 0){
 		  if(args->action == aaFolder && !args->data.folder){
 		      args->action = aaPrcCopy;
@@ -327,12 +333,6 @@ Loop: while(--ac > 0)
 
 		  goto Loop;
 	      }
-#ifdef notdef
-	      else if(strcmp(*av, "nr") == 0){
-		  pine_state->nr_mode = 1;
-		  goto Loop;
-	      }
-#endif /* notdef */
 	      else if(strcmp(*av, "sort") == 0){
 		  if(--ac){
 		      sort = *++av;
@@ -435,6 +435,10 @@ Loop: while(--ac > 0)
 		  pine_state->exit_if_no_pinerc = 1;
 		  goto Loop;
 	      }
+	      else if(strcmp(*av, "version") == 0){
+		  do_version = 1;
+		  goto Loop;
+	      }
 #ifdef	_WINDOWS
 	      else if(strcmp(*av, "registry") == 0){
 		  if(--ac){
@@ -485,11 +489,6 @@ Loop: while(--ac > 0)
 		    case 'k':
 		      do_use_fk = 1;
 		      break;
-#ifdef notdef
-		    case 'a':
-		      anonymous = 1;
-		      break;
-#endif /* notdef */
 		    case 'z':
 		      do_can_suspend = 1;
 		      break;
@@ -752,15 +751,6 @@ Loop: while(--ac > 0)
 	}
     }
 
-    if(anonymous){
-	if(pine_state->nr_mode)
-	  pine_state->anonymous = 1;
-	else{
-	    display_args_err("Can't currently use -a without -nr", NULL, 1);
-	    exit(-1);
-	}
-    }
-
     if(((do_conf ? 1 : 0)+(pinerc_file ? 1 : 0)+(addrbook_file ? 1 : 0)) > 1){
 	display_args_err("May only have one of -conf, -pinerc, and -create_lu",
 			 NULL, 1);
