@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: imap.c,v 4.161 2001/01/19 17:33:06 hubert Exp $";
+static char rcsid[] = "$Id: imap.c,v 4.190 2001/08/17 18:10:21 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -64,6 +64,8 @@ typedef struct _mmlogin_s {
     char	   *user,
 		   *passwd;
     unsigned	    altflag:1;
+    unsigned	    ok_novalidate:1;
+    unsigned	    warned:1;
     STRLIST_S	   *hosts;
     struct _mmlogin_s *next;
 } MMLOGIN_S;
@@ -83,10 +85,14 @@ long  imap_seq_exec PROTO((MAILSTREAM *, char *,
 			   long (*) PROTO((MAILSTREAM *, long, void *)),
 			   void *));
 long  imap_seq_exec_append PROTO((MAILSTREAM *, long, void *));
-char *imap_get_user PROTO((MMLOGIN_S *, STRLIST_S *, int));
+int   imap_get_ssl PROTO((MMLOGIN_S *, STRLIST_S *, int *, int *));
+char *imap_get_user PROTO((MMLOGIN_S *, STRLIST_S *));
 int   imap_get_passwd PROTO((MMLOGIN_S *, char *, char *, STRLIST_S *, int));
-void  imap_set_passwd PROTO((MMLOGIN_S **, char *, char *, STRLIST_S *, int));
+void  imap_set_passwd PROTO((MMLOGIN_S **, char *, char *, STRLIST_S *, int,
+			     int, int));
 int   imap_same_host PROTO((STRLIST_S *, STRLIST_S *));
+int   answer_cert_failure PROTO((int, MSGNO_S *, SCROLL_S *));
+char *ps_get PROTO((size_t));
 #ifdef	PASSFILE
 char  xlate_in PROTO((int));
 char  xlate_out PROTO((char));
@@ -95,6 +101,7 @@ int   read_passfile PROTO((char *, MMLOGIN_S **));
 void  write_passfile PROTO((char *, MMLOGIN_S *));
 int   get_passfile_passwd PROTO((char *, char *, char *, STRLIST_S *, int));
 void  set_passfile_passwd PROTO((char *, char *, char *, STRLIST_S *, int));
+char *get_passfile_user PROTO((char *, STRLIST_S *));
 #endif
 
 
@@ -112,7 +119,18 @@ MM_LIST_S  *mm_list_info;
 /*
  * Local global to hook cached list of host/user/passwd triples to.
  */
-static	MMLOGIN_S	*mm_login_list = NULL;
+static	MMLOGIN_S	*mm_login_list     = NULL;
+static	MMLOGIN_S	*cert_failure_list = NULL;
+
+/*
+ * Instead of storing cached passwords in free storage, store them in this
+ * private space. This makes it easier and more reliable when we want
+ * to zero this space out. We only store passwords here (char *) so we
+ * don't need to worry about alignment.
+ */
+static	char		private_store[512];
+
+static	char *details_cert, *details_host, *details_reason;
 
 
 
@@ -500,9 +518,7 @@ mm_expunged(stream, number)
 	mn_flush_raw(ps_global->msgmap, (long) number);
     }
     else if(stream == ps_global->inbox_stream){
-	long i;
-
-	if(i = mn_raw2m(ps_global->inbox_msgmap, (long) number)){
+	if(mn_raw2m(ps_global->inbox_msgmap, (long) number)){
 	    ps_global->inbox_changed = 1;
 	    ps_global->inbox_expunge_count++;
 	}
@@ -566,28 +582,17 @@ mm_login(mb, user, pwd, trial)
     char   *pwd;
     long    trial;
 {
-    char      prompt[MAX_SCREEN_COLS], *last, *host;
+    char      prompt[1000], *last;
+    char      defuser[NETMAXUSER];
     STRLIST_S hostlist[2];
     HelpType  help ;
-    int       i, j, goal, ugoal, len, rc, q_line, flags;
+    int       i, j, goal, ugoal, len, lastlen, dlen, rc, q_line, flags;
 #define NETMAXPASSWD 100
 
+    dprint(9, (debugfile, "mm_login trial=%ld mb->user=%s\n", trial,
+	       mb->user ? mb->user : "(null)"));
     q_line = -(ps_global->ttyo ? ps_global->ttyo->footer_rows : 3);
 
-    if(ps_global->anonymous) {
-        /*------ Anonymous login mode --------*/
-        if(trial < 1) {
-            strcpy(user, "anonymous");
-            sprintf(pwd, "%s@%s",
-		    ps_global->VAR_USER_ID ? ps_global->VAR_USER_ID : "?",
-		    ps_global->hostname);
-	}
-	else
-	  user[0] = pwd[0] = '\0';
-
-        return;
-    }
-
     /* make sure errors are seen */
     if(ps_global->ttyo)
       flush_status_messages(0);
@@ -608,6 +613,10 @@ mm_login(mb, user, pwd, trial)
      * Initialize user name with either 
      *     1) /user= value in the stream being logged into,
      *  or 2) the user name we're running under.
+     *
+     * Note that VAR_USER_ID is not yet initialized if this login is
+     * the one to access the remote config file. In that case, the user
+     * can supply the username in the config file name with /user=.
      */
     if(trial == 0L){
 	strncpy(user, (*mb->user) ? mb->user :
@@ -616,15 +625,16 @@ mm_login(mb, user, pwd, trial)
 	user[NETMAXUSER-1] = '\0';
 
 	/* try last working password associated with this host. */
-	if(imap_get_passwd(mm_login_list, pwd, user, hostlist, mb->altflag))
+	if(imap_get_passwd(mm_login_list, pwd, user, hostlist,
+	   (mb->sslflag||mb->tlsflag)))
 	  return;
 
 #ifdef	PASSFILE
 	/* check to see if there's a password left over from last session */
-	if(get_passfile_passwd(ps_global->pinerc, pwd,
-			       user, &hostlist[0], mb->altflag)){
+	if(get_passfile_passwd(ps_global->pinerc,pwd,
+			       user,&hostlist[0],(mb->sslflag||mb->tlsflag))){
 	    imap_set_passwd(&mm_login_list, pwd, user,
-			    &hostlist[0], mb->altflag);
+			    &hostlist[0], (mb->sslflag||mb->tlsflag), 0, 0);
 	    return;
 	}
 #endif
@@ -632,50 +642,122 @@ mm_login(mb, user, pwd, trial)
 	/*
 	 * If no explicit user name supplied and we've not logged in
 	 * with our local user name, see if we've visited this
-	 * host before as someone else...
+	 * host before as someone else.
 	 */
-	if(!*mb->user
-	   && (last = imap_get_user(mm_login_list, hostlist, mb->altflag)))
-	  strncpy(user, last, NETMAXUSER);
+	if(!*mb->user &&
+	   (last = imap_get_user(mm_login_list, hostlist))
+#ifdef	PASSFILE
+	   ||
+	   (last = get_passfile_user(ps_global->pinerc, hostlist))
+#endif
+								  ){
+	    strncpy(user, last, NETMAXUSER);
+	    dprint(9, (debugfile, "mm_login: found user=%s\n", user));
+	}
+
+#if !defined(DOS) && !defined(OS2)
+	if(!*mb->user && !*user &&
+	   (last = (ps_global->ui.login && ps_global->ui.login[0])
+				    ? ps_global->ui.login : NULL)
+								 ){
+	    strncpy(user, last, NETMAXUSER);
+	    dprint(9, (debugfile, "mm_login: found user=%s\n", user));
+	}
+#endif
     }
 
     user[NETMAXUSER-1] = '\0';
 
+    /*
+     * Even if we have a user now, user gets a chance to change it.
+     */
     ps_global->mangled_footer = 1;
     if(!*mb->user){
+	char *strtwo, *strthree;
+	char  onebuf[200], twobuf[200], fourbuf[200];
+	int one, two, three, four, five, avail, need;
+
 	help = NO_HELP;
 
-	/* Dress up long hostnames */
-	sprintf(prompt, "%sHOST: ",
-		(!ps_global->ttyo && mb->altflag) ? "+ " : "");
-	len = strlen(prompt);
-	/* leave space for "HOST", "ENTER NAME", and 15 chars for input name */
-	goal = (ps_global->ttyo ? ps_global->ttyo->screen_cols : 80) -
-		(len + 20 +
-		 min(15,
-		     (ps_global->ttyo ? ps_global->ttyo->screen_cols : 80)/5));
-	last = "  ENTER LOGIN NAME: ";
-	if(goal < 9){
-	    last = " LOGIN: ";
-	    if((goal += 13) < 9){
-		last += 1;
-		goal = 0;
+	/*
+	 * Instead of offering user with a value that the user can edit,
+	 * we offer [user] as a default so that the user can type CR to
+	 * use it. Otherwise, the user has to type in whole name.
+	 */
+	strncpy(defuser, user, sizeof(defuser)-1);
+	defuser[sizeof(defuser)-1] = '\0';
+	user[0] = '\0';
+
+	/*
+	 * Need space for "+ HOST: "
+	 *                hostname
+	 *                ENTER LOGIN NAME
+	 *                " [defuser] : "
+	 *                about 15 chars for input
+	 */
+
+	sprintf(onebuf, "%sHOST: ",
+		(!ps_global->ttyo && (mb->sslflag||mb->tlsflag)) ? "+ " : "");
+
+	one = strlen(onebuf);
+	strtwo = mb->host;
+	two = strlen(strtwo);
+	strthree = "  ENTER LOGIN NAME";
+	three = strlen(strthree);
+	sprintf(fourbuf, "%s%.100s%s : ", (*defuser) ? " [" : "",
+					  (*defuser) ? defuser : "",
+					  (*defuser) ? "]" : "");
+	four = strlen(fourbuf);
+	five = max(min(15,
+		       (ps_global->ttyo ? ps_global->ttyo->screen_cols : 80)/5),
+		   6);
+	avail = ps_global->ttyo ? ps_global->ttyo->screen_cols : 80;
+	need = one+two+three+four+five;
+
+	if(avail < need){
+	    /* reduce space three */
+	    strthree = " LOGIN";
+	    need -= (three - strlen(strthree));
+
+	    if(avail < need){
+		/* get two or three more spaces here */
+		sprintf(onebuf, "%sHST:",
+		  (!ps_global->ttyo && (mb->sslflag||mb->tlsflag)) ? "+" : "");
+		need -= (one - strlen(onebuf));
+
+		if(avail < need){
+		    int reduce_to;
+		    /*
+		     * Reduce space for hostname. Best we can do is 6 chars
+		     * with host...
+		     */
+		    reduce_to = (need - avail < two - 6) ? (two-(need-avail))
+							 : 6;
+		    strncpy(twobuf, strtwo, reduce_to-3);
+		    strncpy(twobuf+reduce_to-3, "...", 4);
+		    need -= (two - strlen(twobuf));
+		    strtwo = twobuf;
+
+		    if(avail < need){
+			if(four > 3){
+			    strncpy(fourbuf, " [..] :", 9);
+			    need -= (four - strlen(fourbuf));
+			}
+
+			if(avail < need)
+			  strncpy(fourbuf, ":", 2);
+
+			/*
+			 * If it still doesn't fit, optionally_enter gets
+			 * to worry about it.
+			 */
+		    }
+		}
 	    }
 	}
 
-	if(goal){
-	    for(i = len, j = 0;
-		i < sizeof(prompt) && (prompt[i] = mb->host[j]); i++, j++)
-	      if(i == goal && mb->host[goal+1] && i < sizeof(prompt)){
-		  strcpy(&prompt[i-3], "...");
-		  break;
-	      }
-	}
-	else
-	  i = 0;
-
-	strncpy(&prompt[i], last, sizeof(prompt)-strlen(prompt));
-	prompt[sizeof(prompt)-1] = '\0';
+	sprintf(prompt, "%.200s%.200s%.200s%.200s",
+		onebuf, strtwo, strthree, fourbuf);
 
 	while(1) {
 	    if(ps_global->ttyo)
@@ -688,6 +770,11 @@ mm_login(mb, user, pwd, trial)
 		help = help == NO_HELP ? h_oe_login : NO_HELP;
 		continue;
 	    }
+
+	    /* default */
+	    if(rc == 0 && !*user)
+	      strncpy(user, defuser, NETMAXUSER);
+	      
 	    if(rc != 4)
 	      break;
 	}
@@ -706,12 +793,35 @@ mm_login(mb, user, pwd, trial)
     if(!user[0])
       return;
 
+    /*
+     * Now that we have a user, we can check in the cache again to see
+     * if there is a password there. Try last working password associated
+     * with this host and user.
+     */
+    if(trial == 0L && !*mb->user){
+	if(imap_get_passwd(mm_login_list, pwd, user, hostlist,
+	   (mb->sslflag||mb->tlsflag)))
+	  return;
+
+#ifdef	PASSFILE
+	if(get_passfile_passwd(ps_global->pinerc,pwd,
+			       user,&hostlist[0],(mb->sslflag||mb->tlsflag))){
+	    imap_set_passwd(&mm_login_list, pwd, user,
+			    &hostlist[0], (mb->sslflag||mb->tlsflag), 0, 0);
+	    return;
+	}
+#endif
+    }
+
+    /*
+     * Didn't find password in cache or this isn't the first try. Ask user.
+     */
     help = NO_HELP;
 
     /* Dress up long host/user names */
     /* leave space for "HOST", "USER" "ENTER PWD", 12 for user 6 for pwd */
     sprintf(prompt, "%sHOST: ",
-	    (!ps_global->ttyo && mb->altflag) ? "+ " : "");
+	    (!ps_global->ttyo && (mb->sslflag||mb->tlsflag)) ? "+ " : "");
     len = strlen(prompt);
     goal  = strlen(mb->host);
     ugoal = strlen(user);
@@ -739,11 +849,11 @@ mm_login(mb, user, pwd, trial)
 
     if(goal){
 	sprintf(prompt, "%sHOST: ",
-		(!ps_global->ttyo && mb->altflag) ? "+ " : "");
+		(!ps_global->ttyo && (mb->sslflag||mb->tlsflag)) ? "+ " : "");
 	for(i = len, j = 0;
 	    i < sizeof(prompt) && (prompt[i] = mb->host[j]); i++, j++)
 	  if(i == goal && mb->host[goal+1] && i < sizeof(prompt)){
-	      strcpy(&prompt[i-3], "...");
+	      strncpy(&prompt[i-3], "...", 4);
 	      break;
 	  }
     }
@@ -755,7 +865,7 @@ mm_login(mb, user, pwd, trial)
 	for(i += strlen(&prompt[i]), j = 0;
 	    i < sizeof(prompt) && (prompt[i] = user[j]); i++, j++)
 	  if(j == ugoal && user[ugoal+1] && i < sizeof(prompt)){
-	      strcpy(&prompt[i-3], "...");
+	      strncpy(&prompt[i-3], "...", 4);
 	      break;
 	  }
     }
@@ -785,10 +895,13 @@ mm_login(mb, user, pwd, trial)
     }
 
     /* remember the password for next time */
-    imap_set_passwd(&mm_login_list, pwd, user, hostlist, mb->altflag);
+    if(F_OFF(F_DISABLE_PASSWORD_CACHING,ps_global))
+      imap_set_passwd(&mm_login_list, pwd, user, hostlist,
+		      (mb->sslflag||mb->tlsflag), 0, 0);
 #ifdef	PASSFILE
     /* if requested, remember it on disk for next session */
-    set_passfile_passwd(ps_global->pinerc, pwd, user, hostlist, mb->altflag);
+    set_passfile_passwd(ps_global->pinerc, pwd, user, hostlist,
+			(mb->sslflag||mb->tlsflag));
 #endif
 }
 
@@ -807,7 +920,7 @@ mm_login_alt_cue(mb)
 
 	mark_titlebar_dirty();
 	PutLine0(0, ps_global->ttyo->screen_cols - 1, 
-		 mb->altflag ? "+" : " ");
+		 (mb->sslflag||mb->tlsflag) ? "+" : " ");
 
 	if(lastc){
 	    (void)pico_set_colorp(lastc, PSC_NONE);
@@ -1007,6 +1120,8 @@ mm_flags(stream,number)
 	/* then clear index entry */
 	if(i = mn_raw2m(ps_global->msgmap, (long) number)){
 	    clear_index_cache_ent(i);
+	    if(scores_are_used(SCOREUSE_GET) & SCOREUSE_STATEDEP)
+	      clear_msg_score(stream, i);
 
 	    /* in case number is current, fix titlebar */
 	    if(mn_is_cur(ps_global->msgmap, i))
@@ -1032,25 +1147,23 @@ mm_status(stream, mailbox, status)
     mm_status_result = *status;
 
 #ifdef DEBUG
-    if(ps_global->debug_imap < 3){
-	dprint(0, (debugfile, " Mailbox %s",mailbox));
-	if (status->flags & SA_MESSAGES)
-	  dprint(0, (debugfile, ", %lu messages",status->messages));
+    dprint(2, (debugfile, " Mailbox %s",mailbox));
+    if (status->flags & SA_MESSAGES)
+      dprint(2, (debugfile, ", %lu messages",status->messages));
 
-	if (status->flags & SA_RECENT)
-	  dprint(0, (debugfile, ", %lu recent",status->recent));
+    if (status->flags & SA_RECENT)
+      dprint(2, (debugfile, ", %lu recent",status->recent));
 
-	if (status->flags & SA_UNSEEN)
-	  dprint(0, (debugfile, ", %lu unseen",status->unseen));
+    if (status->flags & SA_UNSEEN)
+      dprint(2, (debugfile, ", %lu unseen",status->unseen));
 
-	if (status->flags & SA_UIDVALIDITY)
-	  dprint(0, (debugfile, ", %lu UID validity", status->uidvalidity));
+    if (status->flags & SA_UIDVALIDITY)
+      dprint(2, (debugfile, ", %lu UID validity", status->uidvalidity));
 
-	if (status->flags & SA_UIDNEXT)
-	  dprint(0, (debugfile, ", %lu next UID",status->uidnext));
+    if (status->flags & SA_UIDNEXT)
+      dprint(2, (debugfile, ", %lu next UID",status->uidnext));
 
-	dprint(0, (debugfile, "\n"));
-    }
+    dprint(2, (debugfile, "\n"));
 #endif
 }
 
@@ -1172,6 +1285,421 @@ pine_tcptimeout(elapsed, sincelast)
 }
 
 
+/*
+ * C-client callback to handle SSL/TLS certificate validation failures
+ *
+ * Returning 0 means error becomes fatal
+ *           Non-zero means certificate problem is ignored and SSL session is
+ *             established
+ *
+ *  We remember the answer and won't re-ask for subsequent open attempts to
+ *  the same hostname.
+ */
+long
+pine_sslcertquery(reason, host, cert)
+    char *reason, *host, *cert;
+{
+    char      tmp[500];
+    char     *unknown = "<unknown>";
+    long      rv = 0L;
+    STRLIST_S hostlist;
+    int       ok_novalidate = 0, warned = 0;
+
+    dprint(1, (debugfile, "sslcertificatequery: host=%s reason=%s cert=%s\n",
+			   host ? host : "?", reason ? reason : "?",
+			   cert ? cert : "?"));
+
+    hostlist.name = host ? host : "";
+    hostlist.next = NULL;
+
+    /*
+     * See if we've been asked about this host before.
+     */
+    if(imap_get_ssl(cert_failure_list, &hostlist, &ok_novalidate, &warned)){
+	/* we were asked before, did we say Yes? */
+	if(ok_novalidate)
+	  rv++;
+
+	if(rv){
+	    dprint(5, (debugfile,
+		       "sslcertificatequery: approved automatically\n"));
+	    return(rv);
+	}
+
+	dprint(1, (debugfile, "sslcertificatequery: we were asked before and said No, so ask again\n"));
+    }
+
+    if(ps_global->ttyo){
+	static struct key ans_certquery_keys[] =
+	   {HELP_MENU,
+	    NULL_MENU,
+	    {"Y","Yes, continue",{MC_YES,1,{'y'}},KS_NONE},
+	    {"D","[Details]",{MC_VIEW_HANDLE,3,{'d',ctrl('M'),ctrl('J')}},KS_NONE},
+	    {"N","No",{MC_NO,1,{'n'}},KS_NONE},
+	    NULL_MENU,
+	    PREVPAGE_MENU,
+	    NEXTPAGE_MENU,
+	    PRYNTTXT_MENU,
+	    WHEREIS_MENU,
+	    FWDEMAIL_MENU,
+	    {"S", "Save", {MC_SAVETEXT,1,{'s'}}, KS_SAVE}};
+	INST_KEY_MENU(ans_certquery_keymenu, ans_certquery_keys);
+	SCROLL_S  sargs;
+	STORE_S  *in_store, *out_store;
+	gf_io_t   pc, gc;
+	HANDLE_S *handles = NULL;
+	int       the_answer = 'n';
+
+	if(!(in_store = so_get(CharStar, NULL, EDIT_ACCESS)) ||
+	   !(out_store = so_get(CharStar, NULL, EDIT_ACCESS)))
+	  goto try_wantto;
+
+	so_puts(in_store, "<HTML><P>There was a failure validating the SSL/TLS certificate for the server");
+
+	so_puts(in_store, "<P><CENTER>");
+	so_puts(in_store, host ? host : unknown);
+	so_puts(in_store, "</CENTER>");
+
+	so_puts(in_store, "<P>The reason for the failure was");
+
+	/* squirrel away details */
+	if(details_host)
+	  fs_give((void **)&details_host);
+	if(details_reason)
+	  fs_give((void **)&details_reason);
+	if(details_cert)
+	  fs_give((void **)&details_cert);
+	
+	details_host   = cpystr(host ? host : unknown);
+	details_reason = cpystr(reason ? reason : unknown);
+	details_cert   = cpystr(cert ? cert : unknown);
+
+	so_puts(in_store, "<P><CENTER>");
+	sprintf(tmp, "%.300s (<A HREF=\"X-Pine-Cert:\">details</A>)",
+		reason ? reason : unknown);
+
+	so_puts(in_store, tmp);
+	so_puts(in_store, "</CENTER>");
+
+	so_puts(in_store, "<P>We have not verified the identity of your server. If you ignore this certificate validation problem and continue, you could end up connecting to an imposter server.");
+
+	so_puts(in_store, "<P>If the certificate validation failure was expected and permanent you may avoid seeing this warning message in the future by adding the option");
+
+	so_puts(in_store, "<P><CENTER>");
+	so_puts(in_store, "/novalidate-cert");
+	so_puts(in_store, "</CENTER>");
+
+	so_puts(in_store, "<P>to the name of the folder you attempted to access.");
+
+	so_puts(in_store, "<P>Answer \"Yes\" to ignore the warning and continue, \"No\" to cancel the open of this folder.");
+
+	so_seek(in_store, 0L, 0);
+	init_handles(&handles);
+	gf_filter_init();
+	gf_link_filter(gf_html2plain,
+		       gf_html2plain_opt(NULL,
+			  ps_global->ttyo->screen_cols,
+			  &handles, GFHP_LOCAL_HANDLES));
+	gf_set_so_readc(&gc, in_store);
+	gf_set_so_writec(&pc, out_store);
+	gf_pipe(gc, pc);
+	gf_clear_so_writec(out_store);
+	gf_clear_so_readc(in_store);
+
+	memset(&sargs, 0, sizeof(SCROLL_S));
+	sargs.text.handles  = handles;
+	sargs.text.text     = so_text(out_store);
+	sargs.text.src      = CharStar;
+	sargs.text.desc     = "help text";
+	sargs.bar.title     = "SSL/TLS CERTIFICATE VALIDATION FAILURE";
+	sargs.proc.tool     = answer_cert_failure;
+	sargs.proc.data.p   = (void *)&the_answer;
+	sargs.keys.menu     = &ans_certquery_keymenu;
+	/* don't want to re-enter c-client */
+	sargs.quell_newmail = 1;
+	setbitmap(sargs.keys.bitmap);
+
+	scrolltool(&sargs);
+
+	if(the_answer == 'y')
+	  rv++;
+
+	ps_global->mangled_screen = 1;
+	ps_global->painted_body_on_startup = 0;
+	ps_global->painted_footer_on_startup = 0;
+	so_give(&in_store);
+	so_give(&out_store);
+	free_handles(&handles);
+	if(details_host)
+	  fs_give((void **)&details_host);
+	if(details_reason)
+	  fs_give((void **)&details_reason);
+	if(details_cert)
+	  fs_give((void **)&details_cert);
+    }
+    else{
+	/*
+	 * If screen hasn't been initialized yet, use want_to.
+	 */
+try_wantto:
+	memset((void *)tmp, 0, sizeof(tmp));
+	strncpy(tmp,
+		reason ? reason : "SSL/TLS certificate validation failure",
+		sizeof(tmp)/2);
+	strncat(tmp, ": Continue anyway ", sizeof(tmp)/2);
+
+	if(want_to(tmp, 'n', 'x', NO_HELP, WT_NORM) == 'y')
+	  rv++;
+    }
+
+    if(rv == 0)
+      q_status_message1(SM_ORDER, 1, 3, "Open of %s cancelled",
+			host ? host : unknown);
+
+    imap_set_passwd(&cert_failure_list, "", "", &hostlist, 0, rv ? 1 : 0, 0);
+
+    dprint(5, (debugfile, "sslcertificatequery: %s\n",
+			   rv ? "approved" : "rejected"));
+
+    return(rv);
+}
+
+
+int
+url_local_certdetails(url)
+    char *url;
+{
+    if(!struncmp(url, "x-pine-cert:", 12)){
+	STORE_S  *store;
+	SCROLL_S  sargs;
+	char     *folded;
+
+	if(!(store = so_get(CharStar, NULL, EDIT_ACCESS))){
+	    q_status_message(SM_ORDER | SM_DING, 7, 10,
+			     "Error allocating space for details.");
+	    return(0);
+	}
+
+	so_puts(store, "Host given by user:\n\n  ");
+	so_puts(store, details_host);
+	so_puts(store, "\n\nReason for failure:\n\n  ");
+	so_puts(store, details_reason);
+	so_puts(store, "\n\nCertificate being verified:\n\n");
+	folded = fold(details_cert, ps_global->ttyo->screen_cols, ps_global->ttyo->screen_cols, 0, 0, "  ", "  ");
+	so_puts(store, folded);
+	fs_give((void **)&folded);
+	so_puts(store, "\n");
+
+	memset(&sargs, 0, sizeof(SCROLL_S));
+	sargs.text.text     = so_text(store);
+	sargs.text.src      = CharStar;
+	sargs.text.desc     = "Details";
+	sargs.bar.title     = "CERT VALIDATION DETAILS";
+	sargs.help.text     = NO_HELP;
+	sargs.help.title    = NULL;
+	sargs.quell_newmail = 1;
+
+	scrolltool(&sargs);
+
+	so_give(&store);	/* free resources associated with store */
+	ps_global->mangled_screen = 1;
+	return(1);
+    }
+
+    return(0);
+}
+
+
+/*
+ * C-client callback to handle SSL/TLS certificate validation failures
+ */
+void
+pine_sslfailure(host, reason, flags)
+    char *host, *reason;
+    unsigned long flags;
+{
+    static struct key ans_certfail_keys[] =
+       {NULL_MENU,
+	NULL_MENU,
+	NULL_MENU,
+	{"C","[Continue]",{MC_YES,3,{'c',ctrl('J'),ctrl('M')}},KS_NONE},
+	NULL_MENU,
+	NULL_MENU,
+	PREVPAGE_MENU,
+	NEXTPAGE_MENU,
+	NULL_MENU,
+	NULL_MENU,
+	NULL_MENU,
+	NULL_MENU};
+    INST_KEY_MENU(ans_certfail_keymenu, ans_certfail_keys);
+    SCROLL_S sargs;
+    STORE_S *store;
+    int      the_answer = 'n', indent, len;
+    char     buf[500];
+    char    *folded;
+    char    *hst = host ? host : "<unknown>";
+    char    *rsn = reason ? reason : "<unknown>";
+    char    *notls = "/notls";
+    STRLIST_S hostlist;
+    int       ok_novalidate = 0, warned = 0;
+
+
+    dprint(1, (debugfile, "sslfailure: host=%s reason=%s\n",
+			   host ? host : "?", reason ? reason : "?"));
+
+    if(flags & NET_SILENT)
+      return;
+
+    hostlist.name = host ? host : "";
+    hostlist.next = NULL;
+
+    /*
+     * See if we've been told about this host before.
+     */
+    if(imap_get_ssl(cert_failure_list, &hostlist, &ok_novalidate, &warned)){
+	/* we were told already */
+	if(warned){
+	    sprintf(buf, "SSL/TLS failure for %.80s: %.300s", hst, rsn);
+	    mm_log(buf, ERROR);
+	    return;
+	}
+    }
+
+    if(!(store = so_get(CharStar, NULL, EDIT_ACCESS)))
+      return;
+
+    strncpy(buf, "There was an SSL/TLS failure for the server", sizeof(buf));
+    folded = fold(buf, ps_global->ttyo->screen_cols, ps_global->ttyo->screen_cols, 0, 0, "", "");
+    so_puts(store, folded);
+    fs_give((void **)&folded);
+    so_puts(store, "\n");
+
+    if((len=strlen(hst)) <= ps_global->ttyo->screen_cols){
+	if((indent=((ps_global->ttyo->screen_cols-len)/2)) > 0)
+	  so_puts(store, repeat_char(indent, SPACE));
+
+	so_puts(store, hst);
+	so_puts(store, "\n");
+    }
+    else{
+	strncpy(buf, hst, sizeof(buf));
+	buf[sizeof(buf)-1] = '\0';
+	folded = fold(buf, ps_global->ttyo->screen_cols, ps_global->ttyo->screen_cols, 0, 0, "", "");
+	so_puts(store, folded);
+	fs_give((void **)&folded);
+    }
+
+    so_puts(store, "\n");
+
+    strncpy(buf, "The reason for the failure was", sizeof(buf));
+    folded = fold(buf, ps_global->ttyo->screen_cols, ps_global->ttyo->screen_cols, 0, 0, "", "");
+    so_puts(store, folded);
+    fs_give((void **)&folded);
+    so_puts(store, "\n");
+
+    if((len=strlen(rsn)) <= ps_global->ttyo->screen_cols){
+	if((indent=((ps_global->ttyo->screen_cols-len)/2)) > 0)
+	  so_puts(store, repeat_char(indent, SPACE));
+
+	so_puts(store, rsn);
+	so_puts(store, "\n");
+    }
+    else{
+	strncpy(buf, rsn, sizeof(buf));
+	buf[sizeof(buf)-1] = '\0';
+	folded = fold(buf, ps_global->ttyo->screen_cols, ps_global->ttyo->screen_cols, 0, 0, "", "");
+	so_puts(store, folded);
+	fs_give((void **)&folded);
+    }
+
+    so_puts(store, "\n");
+
+    strncpy(buf, "This is just an informational message. With the current setup, SSL/TLS will not work. If this error re-occurs every time you run Pine, your current setup is not compatible with the configuration of your mail server. You may want to add the option", sizeof(buf));
+    folded = fold(buf, ps_global->ttyo->screen_cols, ps_global->ttyo->screen_cols, 0, 0, "", "");
+    so_puts(store, folded);
+    fs_give((void **)&folded);
+    so_puts(store, "\n");
+
+    if((len=strlen(notls)) <= ps_global->ttyo->screen_cols){
+	if((indent=((ps_global->ttyo->screen_cols-len)/2)) > 0)
+	  so_puts(store, repeat_char(indent, SPACE));
+
+	so_puts(store, notls);
+	so_puts(store, "\n");
+    }
+    else{
+	strncpy(buf, notls, sizeof(buf));
+	buf[sizeof(buf)-1] = '\0';
+	folded = fold(buf, ps_global->ttyo->screen_cols, ps_global->ttyo->screen_cols, 0, 0, "", "");
+	so_puts(store, folded);
+	fs_give((void **)&folded);
+    }
+
+    so_puts(store, "\n");
+
+    strncpy(buf, "to the name of the mail server you are attempting to access.",
+	    sizeof(buf));
+    folded = fold(buf, ps_global->ttyo->screen_cols, ps_global->ttyo->screen_cols, 0, 0, "", "");
+    so_puts(store, folded);
+    fs_give((void **)&folded);
+    so_puts(store, "\n");
+
+    strncpy(buf, "Type RETURN to continue.", sizeof(buf));
+    folded = fold(buf, ps_global->ttyo->screen_cols, ps_global->ttyo->screen_cols, 0, 0, "", "");
+    so_puts(store, folded);
+    fs_give((void **)&folded);
+
+    memset(&sargs, 0, sizeof(SCROLL_S));
+    sargs.text.text = so_text(store);
+    sargs.text.src  = CharStar;
+    sargs.text.desc = "help text";
+    sargs.bar.title = "SSL/TLS FAILURE";
+    sargs.proc.tool = answer_cert_failure;
+    sargs.proc.data.p = (void *)&the_answer;
+    sargs.keys.menu = &ans_certfail_keymenu;
+    setbitmap(sargs.keys.bitmap);
+    /* don't want to re-enter c-client */
+    sargs.quell_newmail = 1;
+
+    scrolltool(&sargs);
+
+    ps_global->mangled_screen = 1;
+    ps_global->painted_body_on_startup = 0;
+    ps_global->painted_footer_on_startup = 0;
+    so_give(&store);
+
+    imap_set_passwd(&cert_failure_list, "", "", &hostlist, 0, ok_novalidate, 1);
+}
+
+
+int
+answer_cert_failure(cmd, msgmap, sparms)
+    int	       cmd;
+    MSGNO_S   *msgmap;
+    SCROLL_S  *sparms;
+{
+    int rv = 1;
+
+    ps_global->next_screen = SCREEN_FUN_NULL;
+
+    switch(cmd){
+      case MC_YES :
+	*(int *)(sparms->proc.data.p) = 'y';
+	break;
+
+      case MC_NO :
+	*(int *)(sparms->proc.data.p) = 'n';
+	break;
+
+      default:
+	panic("Unexpected command in answer_cert_failure");
+	break;
+    }
+
+    return(rv);
+}
+
+
 char *
 imap_referral(stream, ref, code)
     MAILSTREAM *stream;
@@ -1432,9 +1960,6 @@ pine_block_notify(reason, data)
     int   reason;
     void *data;
 {
-    static int mask = 0;
-    static int deep = 0;
-
     switch(reason){
       case BLOCK_SENSITIVE:		/* sensitive code, disallow alarms */
 	alrm_signal_block();
@@ -1521,26 +2046,61 @@ imap_same_host(hl1, hl2)
 }
 
 
-char *
-imap_get_user(m_list, hostlist, altflag)
+/*
+ * For convenience, we use the same m_list structure (but a different
+ * instance) for storing a list of hosts we've asked the user about when
+ * SSL validation fails. If this function returns TRUE, that means we
+ * have previously asked the user about this host. Ok_novalidate == 1 means
+ * the user said yes, it was ok. Ok_novalidate == 0 means the user
+ * said no. Warned means we warned them already.
+ */
+int
+imap_get_ssl(m_list, hostlist, ok_novalidate, warned)
     MMLOGIN_S *m_list;
     STRLIST_S *hostlist;
-    int	       altflag;
+    int       *ok_novalidate;
+    int       *warned;
 {
     MMLOGIN_S *l;
-    STRLIST_S *h_list;
-    char      *h;
     
     for(l = m_list; l; l = l->next)
-      /* host name and user name must match */
-      if(imap_same_host(l->hosts, hostlist) && altflag == m_list->altflag)
+      if(imap_same_host(l->hosts, hostlist)){
+	  if(ok_novalidate)
+	    *ok_novalidate = l->ok_novalidate;
+
+	  if(warned)
+	    *warned = l->warned;
+
+	  return(TRUE);
+      }
+
+    return(FALSE);
+}
+
+
+/*
+ * Just trying to guess the username the user might want to use on this
+ * host, the user will confirm.
+ */
+char *
+imap_get_user(m_list, hostlist)
+    MMLOGIN_S *m_list;
+    STRLIST_S *hostlist;
+{
+    MMLOGIN_S *l;
+    
+    for(l = m_list; l; l = l->next)
+      if(imap_same_host(l->hosts, hostlist))
 	return(l->user);
 
     return(NULL);
 }
 
 
-
+/*
+ * If we have a matching hostname, username, and altflag in our cache,
+ * attempt to login with the password from the cache.
+ */
 int
 imap_get_passwd(m_list, passwd, user, hostlist, altflag)
     MMLOGIN_S *m_list;
@@ -1549,40 +2109,44 @@ imap_get_passwd(m_list, passwd, user, hostlist, altflag)
     int	       altflag;
 {
     MMLOGIN_S *l;
-    STRLIST_S *h_list;
-    char      *h;
     
+    dprint(9, (debugfile, "imap_get_passwd: user=%s\n",user ? user : "(null)"));
     for(l = m_list; l; l = l->next)
-      /* host name and user name must match */
       if(imap_same_host(l->hosts, hostlist)
-	 && (!*user || !strcmp(user, l->user))
-	 && (*user && l->altflag == altflag)){
-	  strcpy(user, l->user);
-	  strcpy(passwd, l->passwd);
+	 && *user
+	 && !strcmp(user, l->user)
+	 && l->altflag == altflag){
+	  strncpy(passwd, l->passwd, NETMAXPASSWD);
+	  passwd[NETMAXPASSWD-1] = '\0';
+	  dprint(9, (debugfile, "imap_get_passwd: got one: user=%s\n", user));
 	  return(TRUE);
       }
 
+    dprint(9, (debugfile, "imap_get_passwd: no match\n"));
     return(FALSE);
 }
 
 
 
 void
-imap_set_passwd(l, passwd, user, hostlist, altflag)
+imap_set_passwd(l, passwd, user, hostlist, altflag, ok_novalidate, warned)
     MMLOGIN_S **l;
     char       *passwd, *user;
     STRLIST_S  *hostlist;
+    int         altflag, ok_novalidate, warned;
 {
-    STRLIST_S *h_list, **listp;
+    STRLIST_S **listp;
+    size_t      len;
 
+    dprint(9, (debugfile, "imap_set_passwd\n"));
     for(; *l; l = &(*l)->next)
       if(imap_same_host((*l)->hosts, hostlist)
 	 && !strcmp(user, (*l)->user)
 	 && altflag == (*l)->altflag)
-	if(strcmp(passwd, (*l)->passwd)){
-	    fs_give((void **)&(*l)->passwd);
-	    break;
-	}
+	if(strcmp(passwd, (*l)->passwd) ||
+	   (*l)->ok_novalidate != ok_novalidate ||
+	   (*l)->warned != warned)
+	  break;
 	else
 	  return;
 
@@ -1591,16 +2155,22 @@ imap_set_passwd(l, passwd, user, hostlist, altflag)
 	memset(*l, 0, sizeof(MMLOGIN_S));
     }
 
-    if((*l)->passwd)
-      fs_give((void **) &(*l)->passwd);
+    len = strlen(passwd);
+    if(!(*l)->passwd || strlen((*l)->passwd) < len)
+      (*l)->passwd = ps_get(len+1);
 
-    (*l)->passwd = cpystr(passwd);
+    strncpy((*l)->passwd, passwd, len+1);
 
     (*l)->altflag = altflag;
+    (*l)->ok_novalidate = ok_novalidate;
+    (*l)->warned = warned;
 
     if(!(*l)->user)
       (*l)->user = cpystr(user);
 
+    dprint(9, (debugfile, "saving user=%s altflag=%d\n", (*l)->user,
+	       (*l)->altflag));
+
     for( ; hostlist; hostlist = hostlist->next){
 	for(listp = &(*l)->hosts;
 	    *listp && strucmp((*listp)->name, hostlist->name);
@@ -1608,8 +2178,9 @@ imap_set_passwd(l, passwd, user, hostlist, altflag)
 	  ;
 
 	if(!*listp){
-	    *listp = (STRLIST_S *) fs_get(sizeof(STRLIST_S));
+	    *listp = (STRLIST_S *)fs_get(sizeof(STRLIST_S));
 	    (*listp)->name = cpystr(hostlist->name);
+	    dprint(9, (debugfile, "host=%s\n", (*listp)->name));
 	    (*listp)->next = NULL;
 	}
     }
@@ -1622,6 +2193,8 @@ imap_flush_passwd_cache()
 {
     MMLOGIN_S *l;
 
+    memset((void *)private_store, 0, sizeof(private_store));
+
     while(l = mm_login_list){
 	mm_login_list = mm_login_list->next;
 	if(l->user)
@@ -1629,14 +2202,54 @@ imap_flush_passwd_cache()
 
 	free_strlist(&l->hosts);
 
-	if(l->passwd)
-	  fs_give((void **) &l->passwd);
+	fs_give((void **) &l);
+    }
+
+    while(l = cert_failure_list){
+	cert_failure_list = cert_failure_list->next;
+	if(l->user)
+	  fs_give((void **) &l->user);
+
+	free_strlist(&l->hosts);
 
 	fs_give((void **) &l);
     }
 }
 
 
+/*
+ * Mimics fs_get except it only works for char * (no alignment hacks), it
+ * stores in a static array so it is easy to zero it out (that's the whole
+ * purpose), allocations always happen at the end (no free).
+ * If we go past array limit, we don't break, we just use free storage.
+ * Should be awfully rare, though.
+ */
+char *
+ps_get(size)
+    size_t size;
+{
+    static char *last  = private_store;
+    char        *block = NULL;
+
+    /* there is enough space */
+    if(size <= sizeof(private_store) - (last - private_store)){
+	block = last;
+	last += size;
+    }
+    else{
+	q_status_message(SM_ORDER, 1, 3,
+			 "Out of password caching memory, using free storage");
+	dprint(2, (debugfile,
+		   "Out of password caching space in private_store\n"));
+	dprint(2, (debugfile,
+		   "Using free storage instead\n"));
+	block = fs_get(size);
+    }
+
+    return(block);
+}
+
+
 #ifdef	PASSFILE
 /* 
  * Specific functions to support caching username/passwd/host
@@ -1717,6 +2330,8 @@ passfile_name(pinerc, path)
     for(j = 0; path[i] = PASSFILE[j]; i++, j++)
       ;
 
+    dprint(9, (debugfile, "Looking for passfile=%s\n", path));
+
 #if	defined(DOS) || defined(OS2)
     return((stat(path, &sbuf) == 0
 	    && ((sbuf.st_mode & S_IFMT) == S_IFREG))
@@ -1734,7 +2349,7 @@ passfile_name(pinerc, path)
 	return(path);
     }
     else
-      return(NULL);
+	return(NULL);
 #endif
 }
 
@@ -1749,6 +2364,7 @@ read_passfile(pinerc, l)
     int   i, j, n;
     FILE *fp;
 
+    dprint(9, (debugfile, "read_passfile\n"));
     /* if there's no password to read, bag it!! */
     if(!passfile_name(pinerc, tmp) || !(fp = fopen(tmp, "r")))
       return(0);
@@ -1771,13 +2387,14 @@ read_passfile(pinerc, l)
 	      tmp[i++] = '\0';			/* tie off data */
 	}
 
+	dprint(9, (debugfile, "read_passfile: calling imap_set_passwd\n"));
 	if(ui[0] && ui[1] && ui[2]){		/* valid field? */
 	    STRLIST_S hostlist[1];
 	    int	      flags = ui[3] ? atoi(ui[3]) : 0;
 
 	    hostlist[0].name = ui[2];
 	    hostlist[0].next = NULL;
-	    imap_set_passwd(l, ui[0], ui[1], hostlist, flags & 0x01);
+	    imap_set_passwd(l, ui[0], ui[1], hostlist, flags & 0x01, 0, 0);
 	}
     }
 
@@ -1796,6 +2413,7 @@ write_passfile(pinerc, l)
     int   i, n;
     FILE *fp;
 
+    dprint(9, (debugfile, "write_passfile\n"));
     /* if there's no password to read, bag it!! */
     if(!passfile_name(pinerc, tmp) || !(fp = fopen(tmp, "w")))
       return;
@@ -1835,6 +2453,22 @@ get_passfile_passwd(pinerc, passwd, user, hostlist, altflag)
 
 
 
+/*
+ * Just trying to guess the username the user might want to use on this
+ * host, the user will confirm.
+ */
+char *
+get_passfile_user(pinerc, hostlist)
+    char      *pinerc;
+    STRLIST_S *hostlist;
+{
+    return((passfile_cache || read_passfile(pinerc, &passfile_cache))
+	     ? imap_get_user(passfile_cache, hostlist)
+	     : NULL);
+}
+
+
+
 /*
  * set_passfile_passwd - set the password file entry associated with
  *            cache.  The file is assumed to be in the same directory
@@ -1849,7 +2483,7 @@ set_passfile_passwd(pinerc, passwd, user, hostlist, altflag)
     if((passfile_cache || read_passfile(pinerc, &passfile_cache))
        && want_to("Preserve password on DISK for next login", 'y', 'x',
 		  NO_HELP, WT_NORM) == 'y'){
-	imap_set_passwd(&passfile_cache, passwd, user, hostlist, altflag);
+	imap_set_passwd(&passfile_cache, passwd, user, hostlist, altflag, 0, 0);
 	write_passfile(pinerc, passfile_cache);
     }
 }
