@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: screen.c,v 4.130 2000/08/24 19:14:58 hubert Exp $";
+static char rcsid[] = "$Id: screen.c,v 4.133 2001/06/21 17:54:43 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -22,7 +22,7 @@ static char rcsid[] = "$Id: screen.c,v 4.130 2000/08/24 19:14:58 hubert Exp $";
    permission of the University of Washington.
 
    Pine, Pico, and Pilot software and its included text are Copyright
-   1989-2000 by the University of Washington.
+   1989-2001 by the University of Washington.
 
    The full text of our legal notices is contained in the file called
    CPYRIGHT, included with this distribution.
@@ -430,7 +430,7 @@ print_inverted_label(state, m)
     int      col_offset, do_color = 0, skip = 0, len, c;
     char    *lp, *label;
     COLOR_PAIR *name_color = NULL, *label_color = NULL, *lastc = NULL;
-    COLOR_PAIR *defnamec = NULL, *deflabelc = NULL;
+    COLOR_PAIR *deflabelc = NULL;
     struct variable *vars = ps_global->vars;
 
     col_offset = (m->label && (lp=strchr(m->label, ' '))) ? (lp - m->label) : 0;
@@ -505,7 +505,7 @@ print_inverted_label(state, m)
     for(i = m->tl.r; i <= m->br.r; i++)
       for(j = m->tl.c + skip; j < m->br.c; j++){
 	  c = (i == m->lbl.r &&
-	       j - m->lbl.c >= 0 &&
+	       j - m->lbl.c >= 0 &&	/* what's this, has to be true? */
 	       j - m->lbl.c < len) ? label[j - m->lbl.c] : ' ';
 	  if(name_color && col_offset && j == m->lbl.c)
 	    (void)pico_set_colorp(name_color, PSC_NONE);
@@ -1005,7 +1005,7 @@ status_string(stream, mc)
 {
      static char string[2] = {'\0', '\0'};
 
-     if(!mc || ps_global->nr_mode) {
+     if(!mc) {
          string[0] = ' ';
          return(string);
      } 
@@ -1367,7 +1367,7 @@ format_titlebar(start_col)
     /* figure folder_length and what's to be displayed */
     fold_tmp[0] = '\0';
     if(as.style == FileTextPercent || as.style == TextPercent){
-	if(as.style == FileTextPercent && !ps_global->anonymous){
+	if(as.style == FileTextPercent){
 	    char *fmt    = "File: %s%s";
 	    int   avail  = sc - (ver_len + tit_len + loc_len + 10);
 	    fold_len     = strlen(as.folder_name);
@@ -1428,7 +1428,7 @@ format_titlebar(start_col)
 	if(as.stream
 	   && as.stream->mailbox
 	   && mail_valid_net_parse(as.stream->mailbox, &mb)
-	   && mb.altflag)
+	   && (mb.sslflag || mb.tlsflag))
 	  titlebar_line[sc - 1] = '+';
     }
     
